// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package implments

import (
	"LAPP_ACURA_MOM_BACKEND/grmi"
	meta "LAPP_ACURA_MOM_BACKEND/meta/base"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"github.com/go-xorm/xorm"
	"xorm.io/core"
)

/******************************************************************************
 *
 * @Struct Name              : CrossPlanTaskDAOImplement
 *-----------------------------------------------------------------------------
 *
 * @Description              : CrossPlanTask的数据访问对象实现
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
type CrossPlanTaskDAOImplement struct {
	session *xorm.Session
	meta    *grmi.Entity
	userid  string
}

/******************************************************************************
 *
 * @Function Name            : NewCrossPlanTaskDAOImplement
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个CrossPlanTaskDAOImplement实例
 *
 * @Function Parameters      : xorm会话
 *

 * @Return Value             : CrossPlanTaskDAOImplement实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func NewCrossPlanTaskDAOImplement(session *xorm.Session, userid string) *CrossPlanTaskDAOImplement {
	return &CrossPlanTaskDAOImplement{session, &meta.CrossPlanTask, userid}
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/CrossPlanTaskDAO.InsertOne
 *
 ******************************************************************************/
func (impl *CrossPlanTaskDAOImplement) InsertOne(entity *model.CrossPlanTask) error {

	entity.LastUser = impl.userid
	_, err := impl.session.Table(impl.meta.TableName).Insert(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/CrossPlanTaskDAO.DeleteOne
 *
 ******************************************************************************/
func (impl *CrossPlanTaskDAOImplement) DeleteOne(taskNr int) error {
	_, err := impl.session.Table(impl.meta.TableName).ID(core.PK{taskNr}).Delete(impl.meta.Indicator)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}

	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/CrossPlanTaskDAO.SelectOne
 *
 ******************************************************************************/
func (impl *CrossPlanTaskDAOImplement) SelectOne(taskNr int) (*model.CrossPlanTask, error) {
	var data model.CrossPlanTask
	ok, err := impl.session.Table(impl.meta.TableName).ID(core.PK{taskNr}).Get(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	if !ok {
		return nil, nil
	}
	return &data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/CrossPlanTaskDAO.UpdateOne
 *
 ******************************************************************************/
func (impl *CrossPlanTaskDAOImplement) UpdateOne(entity *model.CrossPlanTask) error {

	entity.LastUser = impl.userid
	_, err := impl.session.Table(impl.meta.TableName).Cols(impl.meta.UpdatingMembers...).ID(entity.GetKey()).Update(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/CrossPlanTaskDAO.Insert
 *
 ******************************************************************************/
func (impl *CrossPlanTaskDAOImplement) Insert(entities *[]model.CrossPlanTask) error {
	for i := 0; i < len(*entities); i++ {
		entity := &(*entities)[i]

		entity.LastUser = impl.userid
	}
	_, err := impl.session.Table(impl.meta.TableName).Insert(entities)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/CrossPlanTaskDAO.Delete
 *
 ******************************************************************************/
func (impl *CrossPlanTaskDAOImplement) Delete(entities *[]model.CrossPlanTask) error {
	for _, entity := range *entities {

		_, err := impl.session.Table(impl.meta.TableName).ID(entity.GetKey()).Delete(impl.meta.Indicator)
		if err != nil {
			return grmi.NewDataBaseError(err)
		}
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/CrossPlanTaskDAO.DeleteWhere
 *
 ******************************************************************************/
func (impl *CrossPlanTaskDAOImplement) DeleteWhere(predicates []grmi.Predicate) error {

	parameters := []interface{}{}
	where := ""

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	_, err := session.Delete(impl.meta.Indicator)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/CrossPlanTaskDAO.Select
 *
 ******************************************************************************/
func (impl *CrossPlanTaskDAOImplement) Select(predicates []grmi.Predicate, orderByFields []grmi.Field) ([]model.CrossPlanTask, error) {

	parameters := []interface{}{}
	where := "1 = 1 "

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	if orderByFields != nil {
		for _, field := range orderByFields {
			session = session.OrderBy(field.ColumnName)
		}
	}

	data := make([]model.CrossPlanTask, 0, 10)
	err := session.Find(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	return data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/CrossPlanTaskDAO.SelectCols
 *
 ******************************************************************************/
func (impl *CrossPlanTaskDAOImplement) SelectCols(predicates []grmi.Predicate, orderByFields []grmi.Field, cloumns []string) ([]map[string]interface{}, error) {

	parameters := []interface{}{}
	where := "1 = 1 "

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	if orderByFields != nil {
		for _, field := range orderByFields {
			session = session.OrderBy(field.ColumnName)
		}
	}

	data := make([]map[string]interface{}, 0, 10)
	err := session.Cols(cloumns...).Find(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	return data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/CrossPlanTaskDAO.SelectAndPaging
 *
 ******************************************************************************/
func (impl *CrossPlanTaskDAOImplement) SelectAndPaging(paging *grmi.Paging, predicates []grmi.Predicate, orderByFields []grmi.Field) (grmi.PagingResult, error) {

	parameters := []interface{}{}
	where := "1 = 1 "

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	count, err := session.Count(impl.meta.Indicator)
	if err != nil {
		return grmi.EmptyPagingResult, err
	}

	orderBy := " order by "
	if orderByFields != nil {
		for _, field := range orderByFields {
			if orderBy == " order by " {
				orderBy += field.ColumnName
			} else {
				orderBy += ", " + field.ColumnName
			}
		}
	}
	parameters = append(parameters, paging.Offset(), paging.Size)
	data := make([]model.CrossPlanTask, 0, 10)
	where = " where " + where
	err = impl.session.Table(impl.meta.TableName).SQL("select * from ["+impl.meta.TableName+"]"+where+orderBy+" offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.EmptyPagingResult, err
	}
	return grmi.PagingResult{Records: data, Count: count, PageNumber: paging.Number, PageSize: paging.Size}, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/CrossPlanTaskDAO.Update
 *
 ******************************************************************************/
func (impl *CrossPlanTaskDAOImplement) Update(entities *[]model.CrossPlanTask) error {
	for i := 0; i < len(*entities); i++ {
		entity := &(*entities)[i]

		entity.LastUser = impl.userid
		_, err := impl.session.Table(impl.meta.TableName).Cols(impl.meta.UpdatingMembers...).ID(entity.GetKey()).Update(entity)
		if err != nil {
			return grmi.NewDataBaseError(err)
		}
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/CrossPlanTaskDAO.UpdateWhere
 *
 ******************************************************************************/
func (impl *CrossPlanTaskDAOImplement) UpdateWhere(predicates []grmi.Predicate, entity *model.CrossPlanTask, columns ...string) error {

	parameters := []interface{}{}
	where := ""

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	columns = append(columns, meta.CrossPlanTask_LastUser.Name)
	entity.LastUser = impl.userid
	_, err := impl.session.Table(impl.meta.TableName).Cols(columns...).Update(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}
