// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package implments

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	meta "LAPP_ACURA_MOM_BACKEND/meta/base"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/utils"
	"errors"
	"fmt"
	"github.com/go-xorm/xorm"
	"strconv"
	"strings"
	"time"
	"xorm.io/core"
)

/******************************************************************************
 *
 * @Struct Name              : SnrDAOImplement
 *-----------------------------------------------------------------------------
 *
 * @Description              : Snr的数据访问对象实现
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-19 12:11:47
 *
 ******************************************************************************/
type SnrDAOImplement struct {
	session *xorm.Session
	meta    *grmi.Entity
	plantNr int
	userid  string
}

/******************************************************************************
 *
 * @Function Name            : NewSnrDAOImplement
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个SnrDAOImplement实例
 *
 * @Function Parameters      : xorm会话
 *
 * @Function Parameters      : 基本主键
 *
 * @Return Value             : SnrDAOImplement实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-19 12:11:47
 *
 ******************************************************************************/
func NewSnrDAOImplement(session *xorm.Session, plantNr int, userid string) *SnrDAOImplement {
	return &SnrDAOImplement{session, &meta.Snr, plantNr, userid}
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.InsertOne
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) InsertOne(entity *model.Snr) error {
	entity.PlantNr = impl.plantNr
	entity.LastUser = impl.userid
	_, err := impl.session.Table(impl.meta.TableName).Insert(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.DeleteOne
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) DeleteOne(snrId string) error {
	_, err := impl.session.Table(impl.meta.TableName).ID(core.PK{impl.plantNr, snrId}).Delete(impl.meta.Indicator)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}

	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.SelectOne
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) SelectOne(snrId string) (*model.Snr, error) {
	var data model.Snr
	ok, err := impl.session.Table(impl.meta.TableName).ID(core.PK{impl.plantNr, snrId}).Get(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	if !ok {
		return nil, nil
	}
	return &data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.UpdateOne
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) UpdateOne(entity *model.Snr) error {
	entity.PlantNr = impl.plantNr
	entity.LastUser = impl.userid
	_, err := impl.session.Table(impl.meta.TableName).Cols(impl.meta.UpdatingMembers...).ID(entity.GetKey()).Update(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.Insert
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) Insert(entities *[]model.Snr) error {
	for i := 0; i < len(*entities); i++ {
		entity := &(*entities)[i]
		entity.PlantNr = impl.plantNr
		entity.LastUser = impl.userid
	}
	_, err := impl.session.Table(impl.meta.TableName).Insert(entities)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.Delete
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) Delete(entities *[]model.Snr) error {
	for _, entity := range *entities {
		entity.PlantNr = impl.plantNr
		_, err := impl.session.Table(impl.meta.TableName).ID(entity.GetKey()).Delete(impl.meta.Indicator)
		if err != nil {
			return grmi.NewDataBaseError(err)
		}
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.DeleteWhere
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) DeleteWhere(predicates []grmi.Predicate) error {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.Snr_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	_, err := session.Delete(impl.meta.Indicator)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.Select
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) Select(predicates []grmi.Predicate, orderByFields []grmi.Field) ([]model.Snr, error) {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.Snr_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	session = session.OrderBy(meta.Snr_PlantNr.ColumnName)
	if orderByFields != nil {
		for _, field := range orderByFields {
			session = session.OrderBy(field.ColumnName)
		}
	}

	data := make([]model.Snr, 0, 10)
	err := session.Find(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	return data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.SelectAndPaging
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) SelectAndPaging(paging *grmi.Paging, predicates []grmi.Predicate, orderByFields []grmi.Field) (grmi.PagingResult, error) {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.Snr_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	count, err := session.Count(impl.meta.Indicator)
	if err != nil {
		return grmi.EmptyPagingResult, err
	}

	orderBy := " order by " + meta.Snr_PlantNr.ColumnName
	if orderByFields != nil {
		for _, field := range orderByFields {
			orderBy += ", " + field.ColumnName
		}
	}
	parameters = append(parameters, paging.Offset(), paging.Size)
	data := make([]model.Snr, 0, 10)
	if where != "" {
		where = " where " + where
	}
	err = impl.session.Table(impl.meta.TableName).SQL("select * from "+impl.meta.TableName+where+orderBy+" offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.EmptyPagingResult, err
	}
	return grmi.PagingResult{Records: data, Count: count, PageNumber: paging.Number, PageSize: paging.Size}, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.Update
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) Update(entities *[]model.Snr) error {
	for i := 0; i < len(*entities); i++ {
		entity := &(*entities)[i]
		entity.PlantNr = impl.plantNr
		entity.LastUser = impl.userid
		_, err := impl.session.Table(impl.meta.TableName).Cols(impl.meta.UpdatingMembers...).ID(entity.GetKey()).Update(entity)
		if err != nil {
			return grmi.NewDataBaseError(err)
		}
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.UpdateWhere
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) UpdateWhere(predicates []grmi.Predicate, entity *model.Snr, columns ...string) error {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.Snr_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	columns = append(columns, meta.Snr_LastUser.Name)
	entity.LastUser = impl.userid
	_, err := session.Cols(columns...).Update(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.GetNextSnr
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) GetNextSnr(snr string) (retsnr string, err error) {
	var (
		lastupdtime              time.Time
		y, l_y, d, l_d, wk, l_wk int
		m, l_m                   time.Month
		layout                   []byte
		i, nextsnr               int
	)

	snrtab, err := impl.SelectOne(snr)
	if err != nil {
		return
	}
	if snrtab == nil {
		return "", grmi.NewBusinessError("未查询到流水号规则")
	}
	if snrtab.LastNrCreate == "" {
		nextsnr = snrtab.NextNr
	} else {
		if lastupdtime, err = utils.DateParse(snrtab.LastNrCreate, "YmdHis"); err != nil {
			return
		}
		// 获取下一序列值
		y = time.Now().Year()
		m = time.Now().Month()
		d = time.Now().Day()
		_, wk = time.Now().ISOWeek()
		l_y = lastupdtime.Year()
		l_m = lastupdtime.Month()
		l_d = lastupdtime.Day()
		_, l_wk = lastupdtime.ISOWeek()
		switch strings.ToUpper(snrtab.ResetNrRule) {
		case "DAILY":
			if y == l_y && m == l_m && wk == l_wk && d == l_d {
				nextsnr = snrtab.NextNr
			} else {
				nextsnr = snrtab.StartNr
			}
		case "WEEKLY":
			if y == l_y && m == l_m && wk == l_wk {
				nextsnr = snrtab.NextNr
			} else {
				nextsnr = snrtab.StartNr
			}
		case "MONTHLY":
			if y == l_y && m == l_m {
				nextsnr = snrtab.NextNr
			} else {
				nextsnr = snrtab.StartNr
			}
		case "YEARLY":
			if y == l_y {
				nextsnr = snrtab.NextNr
			} else {
				nextsnr = snrtab.StartNr
			}
		case "NEVER":
			nextsnr = snrtab.NextNr
		default:
			nextsnr = snrtab.NextNr
		}
	}

	// 组合返回值
	layout = []byte(snrtab.Identifierlayout)
	for i = 0; i < len(layout); i++ {
		switch string(layout[i]) {
		case "1": //前缀
			//判断是否开通了映射
			if snrtab.Data1MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrtab.Data1Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					val := v.StdefTyp
					key  := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[snrtab.Prefix]
				if ok {
					retsnr = retsnr + value
				} else {
					retsnr = retsnr + snrtab.Prefix
				}

			} else {
				retsnr = retsnr + snrtab.Prefix
			}
		case "2": //日期变量
			//判断是否开通了映射
			if snrtab.Data2MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrtab.Data2Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					val := v.StdefTyp
					key  := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[utils.Date(time.Now().Unix(), snrtab.DateFormat)]
				if ok {
					retsnr = retsnr + value
				} else {
					retsnr = retsnr + utils.Date(time.Now().Unix(), snrtab.DateFormat)
				}

			} else {

				if snrtab.DateFormat == "Func:GenerateTimeFormatForAcura" {
					dateStr, err := utils.GenerateTimeFormatForAcura(time.Now())
					if err != nil {
						return "", err
					}
					retsnr = retsnr + dateStr
				} else {
					retsnr = retsnr + utils.Date(time.Now().Unix(), snrtab.DateFormat)
				}
			}

		case "3": //中缀

			//判断是否开通了映射
			if snrtab.Data3MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrtab.Data3Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					val := v.StdefTyp
					key := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[snrtab.Infix]
				if ok {
					retsnr = retsnr + value
				} else {
					retsnr = retsnr + snrtab.Infix
				}

			} else {
				retsnr = retsnr + snrtab.Infix
			}

		case "4": //流水号

			var nextnum string
			if snrtab.IdentifierFormat == "CHAR" {
				nextnum = utils.ConvInt2FormatString(nextsnr, snrtab.Length)
			} else {
				nextnum = strconv.Itoa(nextsnr)
			}

			//判断是否开通了映射
			if snrtab.Data4MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrtab.Data4Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					val  := v.StdefTyp
					key := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[nextnum]
				if ok {
					retsnr = retsnr + value
				} else {
					retsnr = retsnr + nextnum
				}

			} else {
				retsnr = retsnr + nextnum
			}
		case "5": //后缀
			//判断是否开通了映射
			if snrtab.Data5MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrtab.Data5Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					val := v.StdefTyp
					key := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[snrtab.Postfix]
				if ok {
					retsnr = retsnr + value
				} else {
					retsnr = retsnr + snrtab.Postfix
				}

			} else {
				retsnr = retsnr + snrtab.Postfix
			}

		}
	}
	//更新下一序列值
	// 处理数值超限
	snrtab.NextNr = nextsnr + snrtab.Step
	snrtab.LastNrCreate = utils.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	snrtab.LastModify = grmi.DateTime(time.Now())
	if snrtab.NextNr > snrtab.EndNr {
		switch strings.ToUpper(snrtab.OverflowHandling) {
		case "NO":
		case "RESET":
			snrtab.NextNr = snrtab.StartNr
		}
	}

	if err = impl.UpdateOne(snrtab); err != nil {
		return
	}

	return
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/services/base/SnrService.GetSessionNextSnr
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) GetSessionNextSnr(user *global.User, session *xorm.Session, snr string) (retsnr string, err error) {
	var (
		lastupdtime              time.Time
		y, l_y, d, l_d, wk, l_wk int
		m, l_m                   time.Month
		layout                   []byte
		i, nextsnr               int
	)

	snrtab, err := impl.SelectOne(snr)
	if err != nil {
		return
	}

	if snrtab.LastNrCreate == "" {
		nextsnr = snrtab.NextNr
	} else {
		if lastupdtime, err = utils.DateParse(snrtab.LastNrCreate, "YmdHis"); err != nil {
			return
		}
		// 获取下一序列值
		y = time.Now().Year()
		m = time.Now().Month()
		d = time.Now().Day()
		_, wk = time.Now().ISOWeek()
		l_y = lastupdtime.Year()
		l_m = lastupdtime.Month()
		l_d = lastupdtime.Day()
		_, l_wk = lastupdtime.ISOWeek()
		switch strings.ToUpper(snrtab.ResetNrRule) {
		case "DAILY":
			if y == l_y && m == l_m && wk == l_wk && d == l_d {
				nextsnr = snrtab.NextNr
			} else {
				nextsnr = snrtab.StartNr
			}
		case "WEEKLY":
			if y == l_y && m == l_m && wk == l_wk {
				nextsnr = snrtab.NextNr
			} else {
				nextsnr = snrtab.StartNr
			}
		case "MONTHLY":
			if y == l_y && m == l_m {
				nextsnr = snrtab.NextNr
			} else {
				nextsnr = snrtab.StartNr
			}
		case "YEARLY":
			if y == l_y {
				nextsnr = snrtab.NextNr
			} else {
				nextsnr = snrtab.StartNr
			}
		case "NEVER":
			nextsnr = snrtab.NextNr
		default:
			nextsnr = snrtab.NextNr
		}
	}

	// 组合返回值
	layout = []byte(snrtab.Identifierlayout)
	for i = 0; i < len(layout); i++ {
		switch string(layout[i]) {
		case "1": //前缀
			//判断是否开通了映射
			if snrtab.Data1MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrtab.Data1Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					key := v.StdefTyp
					val := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[snrtab.Prefix]
				if ok {
					retsnr = retsnr + value
				} else {
					return "", errors.New("前缀映射错误")
				}

			} else {
				retsnr = retsnr + snrtab.Prefix
			}
		case "2": //日期变量
			//判断是否开通了映射
			if snrtab.Data2MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrtab.Data2Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					key := v.StdefTyp
					val := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[utils.Date(time.Now().Unix(), snrtab.DateFormat)]
				if ok {
					retsnr = retsnr + value
				} else {
					return "", errors.New("日期映射错误")
				}

			} else {
				if snrtab.DateFormat == "Func:GenerateTimeFormatForAcura" {
					dateStr, err := utils.GenerateTimeFormatForAcura(time.Now())
					if err != nil {
						return "", err
					}
					retsnr = retsnr + dateStr
				} else {
					retsnr = retsnr + utils.Date(time.Now().Unix(), snrtab.DateFormat)
				}
			}

		case "3": //中缀
			//判断是否开通了映射
			if snrtab.Data3MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrtab.Data3Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					key := v.StdefTyp
					val := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[snrtab.Infix]
				if ok {
					retsnr = retsnr + value
				} else {
					return "", errors.New("前缀映射错误")
				}

			} else {
				retsnr = retsnr + snrtab.Infix
			}

		case "4": //流水号

			var nextnum string
			if snrtab.IdentifierFormat == "CHAR" {
				nextnum = utils.ConvInt2FormatString(nextsnr, snrtab.Length)
			} else {
				nextnum = strconv.Itoa(nextsnr)
			}

			//判断是否开通了映射
			if snrtab.Data4MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrtab.Data4Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					key := v.StdefTyp
					val := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[nextnum]
				if ok {
					retsnr = retsnr + value
				} else {
					return "", errors.New("流水号映射错误")
				}

			} else {
				retsnr = retsnr + nextnum
			}
		case "5": //后缀
			//判断是否开通了映射
			if snrtab.Data5MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrtab.Data5Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					key := v.StdefTyp
					val := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[snrtab.Postfix]
				if ok {
					retsnr = retsnr + value
				} else {
					return "", errors.New("后缀映射错误")
				}

			} else {
				retsnr = retsnr + snrtab.Postfix
			}

		}
	}
	//更新下一序列值
	// 处理数值超限
	snrtab.NextNr = nextsnr + snrtab.Step
	snrtab.LastNrCreate = utils.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	snrtab.LastModify = grmi.DateTime(time.Now())
	if snrtab.NextNr > snrtab.EndNr {
		switch strings.ToUpper(snrtab.OverflowHandling) {
		case "NO":
		case "RESET":
			snrtab.NextNr = snrtab.StartNr
		}
	}

	if err = impl.UpdateOne(snrtab); err != nil {
		return
	}
	return
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.SelectStd
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) SelectStd(dataMap string) ([]model.Stdef, error) {

	session := impl.session.Table("Stdef").Where("StatId = ?",  dataMap)

	data := make([]model.Stdef, 0, 10)
	err := session.Find(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	return data, nil
}

// SelectNexSnrFromCache 从缓存获取下一个序列号
func (impl *SnrDAOImplement) SelectNexSnrFromCache(snrId, date, rule string, startNr, year, month, week int) (int, error) {
	var (
		where string // 查询条件
		parameters []interface{} // 查询参数
	)
	switch rule {
	case model.RESET_RULE_DAYLY:
		where = fmt.Sprintf("%s = ? and %s = ? and %s = ?",
			meta.SnrCache_PlantNr.ColumnName,
			meta.SnrCache_SnrId.ColumnName,
			meta.SnrCache_SnrDate.ColumnName)
		parameters = []interface{}{impl.plantNr, snrId, date}
	case model.RESET_RULE_WEEKLY:
		where = fmt.Sprintf("%s = ? and %s = ? and %s = ? and %s = ?",
			meta.SnrCache_PlantNr.ColumnName,
			meta.SnrCache_SnrId.ColumnName,
			meta.SnrCache_SnrYear.ColumnName,
			meta.SnrCache_SnrWeek.ColumnName)
		parameters = []interface{}{impl.plantNr, snrId, year, week}
	case model.RESET_RULE_MONTHLY:
		where = fmt.Sprintf("%s = ? and %s = ? and %s = ? and %s = ?",
			meta.SnrCache_PlantNr.ColumnName,
			meta.SnrCache_SnrId.ColumnName,
			meta.SnrCache_SnrYear.ColumnName,
			meta.SnrCache_SnrMonth.ColumnName)
		parameters = []interface{}{impl.plantNr, snrId, year, month}
	case model.RESET_RULE_YEARLY:
		where = fmt.Sprintf("%s = ? and %s = ? and %s = ?",
			meta.SnrCache_PlantNr.ColumnName,
			meta.SnrCache_SnrId.ColumnName,
			meta.SnrCache_SnrYear.ColumnName,
		)
		parameters = []interface{}{impl.plantNr, snrId, year}
	default:
		return 0, grmi.NewBusinessError("不支持的规则类型")
	}

	var data model.SnrCache
	ok, err := impl.session.Table(meta.SnrCache.TableName).Where(where, parameters...).Get(&data)
	if err != nil {
		return 0, err
	}
	if !ok {
		return startNr, nil
	}
	return data.NextNr, nil
}

// GetMaxItemNr 获取最大的itemNr
func (impl *SnrDAOImplement) GetMaxItemNr(snrId string) (int, error) {
	where := fmt.Sprintf("%s = ? and %s = ?",
		meta.SnrCache_PlantNr.ColumnName,
		meta.SnrCache_SnrId.ColumnName,
	)
	parameters := []interface{}{impl.plantNr, snrId}
	var snrCache model.SnrCache
	ok, err := impl.session.Table(meta.SnrCache.TableName).Where(where, parameters...).Desc(meta.SnrCache_ItemNr.SortColumnName).Get(&snrCache)
	if err != nil {
		return 0, err
	}
	if !ok {
		return 0, nil
	}
	return snrCache.ItemNr, nil
}

// UpdateSnrCache 更新snrCache
func (impl *SnrDAOImplement) UpdateSnrCache(snrId, rule string, year, month, week, nextSnr int, date time.Time) error {
	var (
		where string // 查询条件
		parameters []interface{} // 查询参数
	)
	switch rule {
	case model.RESET_RULE_DAYLY:
		where = fmt.Sprintf("%s = ? and %s = ? and %s = ?",
			meta.SnrCache_PlantNr.ColumnName,
			meta.SnrCache_SnrId.ColumnName,
			meta.SnrCache_SnrDate.ColumnName)
		parameters = []interface{}{impl.plantNr, snrId, date.Format(grmi.DateOutFormat)}
	case model.RESET_RULE_WEEKLY:
		where = fmt.Sprintf("%s = ? and %s = ? and %s = ? and %s = ?",
			meta.SnrCache_PlantNr.ColumnName,
			meta.SnrCache_SnrId.ColumnName,
			meta.SnrCache_SnrYear.ColumnName,
			meta.SnrCache_SnrWeek.ColumnName)
		parameters = []interface{}{impl.plantNr, snrId, year, week}
	case model.RESET_RULE_MONTHLY:
		where = fmt.Sprintf("%s = ? and %s = ? and %s = ? and %s = ?",
			meta.SnrCache_PlantNr.ColumnName,
			meta.SnrCache_SnrId.ColumnName,
			meta.SnrCache_SnrYear.ColumnName,
			meta.SnrCache_SnrMonth.ColumnName)
		parameters = []interface{}{impl.plantNr, snrId, year, month}
	case model.RESET_RULE_YEARLY:
		where = fmt.Sprintf("%s = ? and %s = ? and %s = ?",
			meta.SnrCache_PlantNr.ColumnName,
			meta.SnrCache_SnrId.ColumnName,
			meta.SnrCache_SnrYear.ColumnName,
		)
		parameters = []interface{}{impl.plantNr, snrId, year}
	default:
		return grmi.NewBusinessError("不支持的规则类型")
	}
	var data model.SnrCache
	ok, err := impl.session.Table(meta.SnrCache.TableName).Where(where, parameters...).Get(&data)
	if err != nil {
		return err
	}
	if !ok {
		maxItemNr, err := impl.GetMaxItemNr(snrId)
		if err != nil {
			return err
		}
		snrCache := model.SnrCache{
			PlantNr: impl.plantNr,
			SnrId: snrId,
			ItemNr: maxItemNr+1,
			SnrDate: grmi.Date(date),
			SnrYear: year,
			SnrMonth: month,
			SnrDay: date.Day(),
			SnrWeek: week,
			NextNr: nextSnr,
		}
		_, err = impl.session.Table(meta.SnrCache.TableName).Insert(&snrCache)
		if err != nil {
			return grmi.NewDataBaseError(err)
		}
		return nil
	} else {
		data.SnrDate = grmi.Date(date)
		data.LastModify = grmi.DateTime(time.Now())
		data.LastUser = impl.userid
		data.NextNr = nextSnr
		data.SnrYear = year
		data.SnrMonth = month
		data.SnrDay = date.Day()
		data.SnrWeek = week
		_, err = impl.session.Table(meta.SnrCache.TableName).Cols(meta.SnrCache.UpdatingMembers...).ID(data.GetKey()).Update(&data)
		if err != nil {
			return grmi.NewDataBaseError(err)
		}
		return nil
	}
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/base/SnrDAO.GetNextSnrWithTime
 *
 ******************************************************************************/
func (impl *SnrDAOImplement) GetNextSnrWithTime(snr string, t time.Time) (returnSnr string, err error) {
	year, month, _ := t.Date()
	_, week := t.ISOWeek()
	date := t.Format(grmi.DateOutFormat)
	snrData, err := impl.SelectOne(snr)
	if err != nil {
		return
	}
	if snrData == nil {
		return "", grmi.NewBusinessError("未查询到流水号规则")
	}
	nextSnr, err := impl.SelectNexSnrFromCache(snr, date, snrData.ResetNrRule, snrData.StartNr, year, int(month), week)
	if err != nil {
		return "", err
	}
	// 组合返回值
	layout := []byte(snrData.Identifierlayout)
	for i := 0; i < len(layout); i++ {
		switch string(layout[i]) {
		case "1": //前缀
			//判断是否开通了映射
			if snrData.Data1MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrData.Data1Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					val := v.StdefTyp
					key  := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[snrData.Prefix]
				if ok {
					returnSnr = returnSnr + value
				} else {
					returnSnr = returnSnr + snrData.Prefix
				}

			} else {
				returnSnr = returnSnr + snrData.Prefix
			}
		case "2": //日期变量
			//判断是否开通了映射
			if snrData.Data2MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrData.Data2Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					val := v.StdefTyp
					key  := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[utils.Date(t.Unix(), snrData.DateFormat)]
				if ok {
					returnSnr = returnSnr + value
				} else {
					returnSnr = returnSnr + utils.Date(t.Unix(), snrData.DateFormat)
				}

			} else {

				if snrData.DateFormat == "Func:GenerateTimeFormatForAcura" {
					dateStr, err := utils.GenerateTimeFormatForAcura(t)
					if err != nil {
						return "", err
					}
					returnSnr = returnSnr + dateStr
				} else {
					returnSnr = returnSnr + utils.Date(t.Unix(), snrData.DateFormat)
				}
			}

		case "3": //中缀

			//判断是否开通了映射
			if snrData.Data3MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrData.Data3Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					val := v.StdefTyp
					key := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[snrData.Infix]
				if ok {
					returnSnr = returnSnr + value
				} else {
					returnSnr = returnSnr + snrData.Infix
				}

			} else {
				returnSnr = returnSnr + snrData.Infix
			}

		case "4": //流水号

			var nextnum string
			if snrData.IdentifierFormat == "CHAR" {
				nextnum = utils.ConvInt2FormatString(nextSnr, snrData.Length)
			} else {
				nextnum = strconv.Itoa(nextSnr)
			}

			//判断是否开通了映射
			if snrData.Data4MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrData.Data4Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					val  := v.StdefTyp
					key := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[nextnum]
				if ok {
					returnSnr = returnSnr + value
				} else {
					returnSnr = returnSnr + nextnum
				}

			} else {
				returnSnr = returnSnr + nextnum
			}
		case "5": //后缀
			//判断是否开通了映射
			if snrData.Data5MappingToggle {
				//查询对应的key Map
				stdData, err := impl.SelectStd(snrData.Data5Mapper)
				if err != nil {
					return "", err
				}
				stdMap := make(map[string]string)
				for _, v := range stdData {
					val := v.StdefTyp
					key := v.Bez
					stdMap[key] = val
				}
				value, ok := stdMap[snrData.Postfix]
				if ok {
					returnSnr = returnSnr + value
				} else {
					returnSnr = returnSnr + snrData.Postfix
				}

			} else {
				returnSnr = returnSnr + snrData.Postfix
			}

		}
	}
	//更新下一序列值
	updateNextSnr := nextSnr + snrData.Step
	if updateNextSnr > snrData.EndNr {
		switch strings.ToUpper(snrData.OverflowHandling) {
		case "NO":
		case "RESET":
			updateNextSnr = snrData.StartNr
		}
	}
	err = impl.UpdateSnrCache(snr, snrData.ResetNrRule, year, int(month), week, updateNextSnr, t)
	return
}