// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package implments

import (
	"LAPP_ACURA_MOM_BACKEND/grmi"
	meta "LAPP_ACURA_MOM_BACKEND/meta/jit"
	model "LAPP_ACURA_MOM_BACKEND/models/jit"
	"errors"
	"fmt"
	"github.com/go-xorm/xorm"
	"xorm.io/core"
)

/******************************************************************************
 *
 * @Struct Name              : PackOrderStatusDAOImplement
 *-----------------------------------------------------------------------------
 *
 * @Description              : PackOrderStatus的数据访问对象实现
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
type PackOrderStatusDAOImplement struct {
	session *xorm.Session
	meta    *grmi.Entity
	plantNr int
	userid  string
}

/******************************************************************************
 *
 * @Function Name            : NewPackOrderStatusDAOImplement
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个PackOrderStatusDAOImplement实例
 *
 * @Function Parameters      : xorm会话
 *
 * @Function Parameters      : 基本主键
 *
 * @Return Value             : PackOrderStatusDAOImplement实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func NewPackOrderStatusDAOImplement(session *xorm.Session, plantNr int, userid string) *PackOrderStatusDAOImplement {
	return &PackOrderStatusDAOImplement{session, &meta.PackOrderStatus, plantNr, userid}
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.InsertOne
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) InsertOne(entity *model.PackOrderStatus) error {
	entity.PlantNr = impl.plantNr
	entity.LastUser = impl.userid
	_, err := impl.session.Table(impl.meta.TableName).Insert(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.DeleteOne
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) DeleteOne(packOrderId string) error {
	_, err := impl.session.Table(impl.meta.TableName).ID(core.PK{impl.plantNr, packOrderId}).Delete(impl.meta.Indicator)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}

	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.SelectOne
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) SelectOne(packOrderId string) (*model.PackOrderStatus, error) {
	var data model.PackOrderStatus
	ok, err := impl.session.Table(impl.meta.TableName).ID(core.PK{impl.plantNr, packOrderId}).Get(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	if !ok {
		return nil, nil
	}
	return &data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.UpdateOne
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) UpdateOne(entity *model.PackOrderStatus) error {
	entity.PlantNr = impl.plantNr
	entity.LastUser = impl.userid
	_, err := impl.session.Table(impl.meta.TableName).Cols(impl.meta.UpdatingMembers...).ID(entity.GetKey()).Update(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.Insert
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) Insert(entities *[]model.PackOrderStatus) error {
	for i := 0; i < len(*entities); i++ {
		entity := &(*entities)[i]
		entity.PlantNr = impl.plantNr
		entity.LastUser = impl.userid
	}
	_, err := impl.session.Table(impl.meta.TableName).Insert(entities)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.Delete
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) Delete(entities *[]model.PackOrderStatus) error {
	for _, entity := range *entities {
		entity.PlantNr = impl.plantNr
		_, err := impl.session.Table(impl.meta.TableName).ID(entity.GetKey()).Delete(impl.meta.Indicator)
		if err != nil {
			return grmi.NewDataBaseError(err)
		}
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.DeleteWhere
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) DeleteWhere(predicates []grmi.Predicate) error {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.PackOrderStatus_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	_, err := session.Delete(impl.meta.Indicator)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.Select
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) Select(predicates []grmi.Predicate, orderByFields []grmi.Field) ([]model.PackOrderStatus, error) {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.PackOrderStatus_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	session = session.OrderBy(meta.PackOrderStatus_PlantNr.ColumnName)
	if orderByFields != nil {
		for _, field := range orderByFields {
			session = session.OrderBy(field.ColumnName)
		}
	}

	data := make([]model.PackOrderStatus, 0, 10)
	err := session.Find(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	return data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.SelectAndPaging
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) SelectAndPaging(paging *grmi.Paging, predicates []grmi.Predicate, orderByFields []grmi.Field) (grmi.PagingResult, error) {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.PackOrderStatus_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	count, err := session.Count(impl.meta.Indicator)
	if err != nil {
		return grmi.EmptyPagingResult, err
	}

	orderBy := " order by " + meta.PackOrderStatus_PlantNr.ColumnName
	if orderByFields != nil {
		for _, field := range orderByFields {
			if orderBy == " order by " {
				orderBy += field.ColumnName
			} else {
				orderBy += ", " + field.ColumnName
			}
		}
	}
	parameters = append(parameters, paging.Offset(), paging.Size)
	data := make([]model.PackOrderStatus, 0, 10)
	where = " where " + where
	err = impl.session.Table(impl.meta.TableName).SQL("select * from ["+impl.meta.TableName+"]"+where+orderBy+" offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.EmptyPagingResult, err
	}
	return grmi.PagingResult{Records: data, Count: count, PageNumber: paging.Number, PageSize: paging.Size}, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.Update
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) Update(entities *[]model.PackOrderStatus) error {
	for i := 0; i < len(*entities); i++ {
		entity := &(*entities)[i]
		entity.PlantNr = impl.plantNr
		entity.LastUser = impl.userid
		_, err := impl.session.Table(impl.meta.TableName).Cols(impl.meta.UpdatingMembers...).ID(entity.GetKey()).Update(entity)
		if err != nil {
			return grmi.NewDataBaseError(err)
		}
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.UpdateWhere
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) UpdateWhere(predicates []grmi.Predicate, entity *model.PackOrderStatus, columns ...string) error {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.PackOrderStatus_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	columns = append(columns, meta.PackOrderStatus_LastUser.Name)
	entity.LastUser = impl.userid
	_, err := session.Cols(columns...).Update(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.SelectUnFlushBackAndUpdateSign
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) SelectUnFlushBackAndUpdateSign(now string) (int64, error) {
	where := fmt.Sprintf("%s = ? and %s = ? and %s = ? and %s < ?",
		meta.PackOrderStatus_PlantNr.ColumnName,
		meta.PackOrderStatus_Status.ColumnName,
		meta.PackOrderStatus_BackflushedToggle.ColumnName,
		meta.PackOrderStatus_LastModify.ColumnName)
	parameters := []interface{}{impl.plantNr, model.PACK_STATUS_CLOSED, 0, now}

	count, err := impl.session.Table(impl.meta.TableName).Where(where, parameters...).Count()
	if err != nil {
		return 0, err
	}
	if count != 0 {
		updateData := make(map[string]int)
		updateData[meta.PackOrderStatus_BackflushedToggle.Name] = 1
		updateCount, err := impl.session.Table(impl.meta.TableName).Where(where, parameters...).Cols(meta.PackOrderStatus_BackflushedToggle.ColumnName).Update(&updateData)
		if err != nil {
			return 0, err
		}
		if updateCount != count {
			return 0, errors.New("更新数量和查询数量不一致:" + err.Error())
		}
	}
	return count, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/PackOrderStatusDAO.UpdateBackFlushFlag
 *
 ******************************************************************************/
func (impl *PackOrderStatusDAOImplement) UpdateBackFlushFlag(packOrderIdLi []string) (int64, error) {
	where := fmt.Sprintf("%s = ? and %s = ?",
		meta.PackOrderStatus_PlantNr.ColumnName,
		meta.PackOrderStatus_BackflushedToggle.ColumnName)
	parameters := []interface{}{impl.plantNr, 0}
	inStr := fmt.Sprintf(" and %s in ( ", meta.PackOrderStatus_PackOrderId.ColumnName)
	for index, packOrderId := range packOrderIdLi {
		if index != 0 {
			inStr += ","
		}
		inStr += `'` + packOrderId + `'`
	}
	inStr += " ) "
	where += inStr
	updateMap := make(map[string]bool)
	updateMap[meta.PackOrderStatus_BackflushedToggle.Name] = true
	count, err := impl.session.Table(impl.meta.TableName).Where(where, parameters...).Cols(meta.PackOrderStatus_BackflushedToggle.ColumnName).Update(&updateMap)
	return count, err
}
