// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package implments

import (
	"LAPP_ACURA_MOM_BACKEND/grmi"
	meta "LAPP_ACURA_MOM_BACKEND/meta/jit"
	model "LAPP_ACURA_MOM_BACKEND/models/jit"
	"fmt"
	"github.com/go-xorm/xorm"
	"xorm.io/core"
)

/******************************************************************************
 *
 * @Struct Name              : ShipOrderDAOImplement
 *-----------------------------------------------------------------------------
 *
 * @Description              : ShipOrder的数据访问对象实现
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
type ShipOrderDAOImplement struct {
	session *xorm.Session
	meta    *grmi.Entity
	plantNr int
	userid  string
}

/******************************************************************************
 *
 * @Function Name            : NewShipOrderDAOImplement
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个ShipOrderDAOImplement实例
 *
 * @Function Parameters      : xorm会话
 *
 * @Function Parameters      : 基本主键
 *
 * @Return Value             : ShipOrderDAOImplement实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func NewShipOrderDAOImplement(session *xorm.Session, plantNr int, userid string) *ShipOrderDAOImplement {
	return &ShipOrderDAOImplement{session, &meta.ShipOrder, plantNr, userid}
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.InsertOne
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) InsertOne(entity *model.ShipOrder) error {
	entity.PlantNr = impl.plantNr
	entity.LastUser = impl.userid
	_, err := impl.session.Table(impl.meta.TableName).Insert(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.DeleteOne
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) DeleteOne(shipOrderId string) error {
	_, err := impl.session.Table(impl.meta.TableName).ID(core.PK{impl.plantNr, shipOrderId}).Delete(impl.meta.Indicator)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}

	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.SelectOne
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) SelectOne(shipOrderId string) (*model.ShipOrder, error) {
	var data model.ShipOrder
	ok, err := impl.session.Table(impl.meta.TableName).ID(core.PK{impl.plantNr, shipOrderId}).Get(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	if !ok {
		return nil, nil
	}
	return &data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.UpdateOne
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) UpdateOne(entity *model.ShipOrder) error {
	entity.PlantNr = impl.plantNr
	entity.LastUser = impl.userid
	_, err := impl.session.Table(impl.meta.TableName).Cols(impl.meta.UpdatingMembers...).ID(entity.GetKey()).Update(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.Insert
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) Insert(entities *[]model.ShipOrder) error {
	for i := 0; i < len(*entities); i++ {
		entity := &(*entities)[i]
		entity.PlantNr = impl.plantNr
		entity.LastUser = impl.userid
	}
	_, err := impl.session.Table(impl.meta.TableName).Insert(entities)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.Delete
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) Delete(entities *[]model.ShipOrder) error {
	for _, entity := range *entities {
		entity.PlantNr = impl.plantNr
		_, err := impl.session.Table(impl.meta.TableName).ID(entity.GetKey()).Delete(impl.meta.Indicator)
		if err != nil {
			return grmi.NewDataBaseError(err)
		}
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.DeleteWhere
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) DeleteWhere(predicates []grmi.Predicate) error {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.ShipOrder_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	_, err := session.Delete(impl.meta.Indicator)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.Select
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) Select(predicates []grmi.Predicate, orderByFields []grmi.Field) ([]model.ShipOrder, error) {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.ShipOrder_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	session = session.OrderBy(meta.ShipOrder_PlantNr.ColumnName)
	if orderByFields != nil {
		for _, field := range orderByFields {
			session = session.OrderBy(field.ColumnName)
		}
	}

	data := make([]model.ShipOrder, 0, 10)
	err := session.Find(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	return data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.SelectAndPaging
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) SelectAndPaging(paging *grmi.Paging, predicates []grmi.Predicate, orderByFields []grmi.Field) (grmi.PagingResult, error) {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.ShipOrder_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	count, err := session.Count(impl.meta.Indicator)
	if err != nil {
		return grmi.EmptyPagingResult, err
	}

	orderBy := " order by " + meta.ShipOrder_PlantNr.ColumnName
	if orderByFields != nil {
		for _, field := range orderByFields {
			if orderBy == " order by " {
				orderBy += field.ColumnName
			} else {
				orderBy += ", " + field.ColumnName
			}
		}
	}
	parameters = append(parameters, paging.Offset(), paging.Size)
	data := make([]model.ShipOrder, 0, 10)
	where = " where " + where
	err = impl.session.Table(impl.meta.TableName).SQL("select * from ["+impl.meta.TableName+"]"+where+orderBy+" offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.EmptyPagingResult, err
	}
	return grmi.PagingResult{Records: data, Count: count, PageNumber: paging.Number, PageSize: paging.Size}, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.Update
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) Update(entities *[]model.ShipOrder) error {
	for i := 0; i < len(*entities); i++ {
		entity := &(*entities)[i]
		entity.PlantNr = impl.plantNr
		entity.LastUser = impl.userid
		_, err := impl.session.Table(impl.meta.TableName).Cols(impl.meta.UpdatingMembers...).ID(entity.GetKey()).Update(entity)
		if err != nil {
			return grmi.NewDataBaseError(err)
		}
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.UpdateWhere
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) UpdateWhere(predicates []grmi.Predicate, entity *model.ShipOrder, columns ...string) error {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.ShipOrder_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	columns = append(columns, meta.ShipOrder_LastUser.Name)
	entity.LastUser = impl.userid
	_, err := session.Cols(columns...).Update(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.SelectUnAccomplishedShipOrder
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) SelectUnAccomplishedShipOrder() ([]interface{}, error) {
	where := fmt.Sprintf("%s = ? and %s < ?", meta.ShipOrder_PlantNr.ColumnName, meta.ShipOrderStatus_Status.ColumnName)
	parameters := []interface{}{impl.plantNr, model.PACK_STATUS_CLOSED}
	condition := fmt.Sprintf("%s = %s and %s = %s",
		meta.ShipOrder_PlantNr.ColumnName,
		meta.ShipOrderStatus_PlantNr.ColumnName,
		meta.ShipOrder_ShipOrderId.ColumnName,
		meta.ShipOrderStatus_ShipOrderId.ColumnName)
	result := make([]interface{}, 0)
	err := impl.session.Table(impl.meta.TableName).Join("LEFT", meta.ShipOrderStatus.TableName, condition).
		Where(where, parameters...).Cols(meta.ShipOrder_ShipOrderId.ColumnName).Find(&result)
	return result, err
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.SelectOne
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) SelectOneWithStatus(shipOrderId string) (*model.ShipOrderWithStatus, error) {
	condition := fmt.Sprintf("%s = %s and %s = %s",
		meta.ShipOrder_PlantNr.ColumnName,
		meta.ShipOrderStatus_PlantNr.ColumnName,
		meta.ShipOrder_ShipOrderId.ColumnName,
		meta.ShipOrderStatus_ShipOrderId.ColumnName)
	where := fmt.Sprintf("%s = ? and %s = ?", meta.ShipOrder_PlantNr.ColumnName, meta.ShipOrder_ShipOrderId.ColumnName)
	parameters := []interface{}{impl.plantNr, shipOrderId}
	var data model.ShipOrderWithStatus
	ok, err := impl.session.Table(impl.meta.TableName).Join("LEFT", meta.ShipOrderStatus.TableName, condition).Where(where, parameters...).Get(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	if !ok {
		return nil, nil
	}
	return &data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.SelectAndPaging
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) SelectAndPagingWithStatus(paging *grmi.Paging, predicates []grmi.Predicate, orderByFields []grmi.Field, status int, formalStatus int) (grmi.PagingResult, error) {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.ShipOrder_PlantNr.ColumnName)
	condition := fmt.Sprintf("%s = %s and %s = %s",
		meta.ShipOrder_PlantNr.ColumnName,
		meta.ShipOrderStatus_PlantNr.ColumnName,
		meta.ShipOrder_ShipOrderId.ColumnName,
		meta.ShipOrderStatus_ShipOrderId.ColumnName)
	jointStr := fmt.Sprintf(" left join %s on %s ", meta.ShipOrderStatus.TableName, condition)
	session := impl.session.Table(impl.meta.TableName).Join("LEFT", meta.ShipOrderStatus.TableName, condition)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}
	if status == 0 {
		where += fmt.Sprintf(" and %s < ? ", meta.ShipOrderStatus_Status.ColumnName)
		parameters = append(parameters, model.PACK_STATUS_CLOSED)
		session = session.Where(fmt.Sprintf(" %s < ? ", meta.ShipOrderStatus_Status.ColumnName), model.PACK_STATUS_CLOSED)
	} else {
		where += fmt.Sprintf(" and %s = ? ", meta.ShipOrderStatus_Status.ColumnName)
		parameters = append(parameters, status)
		session = session.Where(fmt.Sprintf(" %s = ? ", meta.ShipOrderStatus_Status.ColumnName), status)
	}
	if formalStatus != 0 {
		where += fmt.Sprintf(" and %s = ? ", meta.ShipOrderStatus_Status1.ColumnName)
		parameters = append(parameters, formalStatus)
		session = session.Where(fmt.Sprintf(" %s = ? ", meta.ShipOrderStatus_Status1.ColumnName), formalStatus)
	}
	count, err := session.Count(impl.meta.Indicator)
	if err != nil {
		return grmi.EmptyPagingResult, err
	}

	orderBy := " order by " + meta.ShipOrder_PlantNr.ColumnName
	if orderByFields != nil {
		for _, field := range orderByFields {
			if orderBy == " order by " {
				orderBy += field.ColumnName
			} else {
				orderBy += ", " + field.ColumnName
			}
		}
	}
	parameters = append(parameters, paging.Offset(), paging.Size)
	data := make([]model.ShipOrderWithStatus, 0, 10)
	where = " where " + where
	err = impl.session.Table(impl.meta.TableName).SQL("select * from ["+impl.meta.TableName+"]"+jointStr+where+orderBy+" offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.EmptyPagingResult, err
	}
	return grmi.PagingResult{Records: data, Count: count, PageNumber: paging.Number, PageSize: paging.Size}, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.SelectWithStatus
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) SelectWithStatus(predicates []grmi.Predicate, orderByFields []grmi.Field, status int, formalStatus int) ([]model.ShipOrderWithStatus, error) {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.ShipOrder_PlantNr.ColumnName)
	condition := fmt.Sprintf("%s = %s and %s = %s",
		meta.ShipOrder_PlantNr.ColumnName,
		meta.ShipOrderStatus_PlantNr.ColumnName,
		meta.ShipOrder_ShipOrderId.ColumnName,
		meta.ShipOrderStatus_ShipOrderId.ColumnName)
	session := impl.session.Table(impl.meta.TableName).Join("LEFT", meta.ShipOrderStatus.TableName, condition)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}
	if status != 0 {
		session = session.Where(fmt.Sprintf("%s = ?", meta.ShipOrderStatus_Status.ColumnName), status)
	} else {
		session = session.Where(fmt.Sprintf(" %s < ? ", meta.ShipOrderStatus_Status.ColumnName), model.PACK_STATUS_CLOSED)
	}
	if formalStatus != 0 {
		session = session.Where(fmt.Sprintf("%s = ?", meta.ShipOrderStatus_Status1.ColumnName), formalStatus)
	}
	session = session.OrderBy(meta.ShipOrder_PlantNr.ColumnName)
	if orderByFields != nil {
		for _, field := range orderByFields {
			session = session.OrderBy(field.ColumnName)
		}
	}

	data := make([]model.ShipOrderWithStatus, 0, 10)
	err := session.Find(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	return data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/jit/ShipOrderDAO.SelectOrderWithStatus
 *
 ******************************************************************************/
func (impl *ShipOrderDAOImplement) SelectOrderWithStatus(predicates []grmi.Predicate, orderByFields []grmi.Field) ([]model.ShipOrderWithStatus, error) {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.ShipOrder_PlantNr.ColumnName)

	condition := fmt.Sprintf("%s = %s and %s = %s",
		meta.ShipOrder_PlantNr.ColumnName,
		meta.ShipOrderStatus_PlantNr.ColumnName,
		meta.ShipOrder_ShipOrderId.ColumnName,
		meta.ShipOrderStatus_ShipOrderId.ColumnName)
	session := impl.session.Table(impl.meta.TableName).Join("LEFT", meta.ShipOrderStatus.TableName, condition)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	session = session.OrderBy(meta.ShipOrder_PlantNr.ColumnName)
	if orderByFields != nil {
		for _, field := range orderByFields {
			session = session.OrderBy(field.ColumnName)
		}
	}

	data := make([]model.ShipOrderWithStatus, 0, 10)
	err := session.Find(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	return data, nil
}