// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package implments

import (
	"LAPP_ACURA_MOM_BACKEND/grmi"
	meta "LAPP_ACURA_MOM_BACKEND/meta/pln"
	model "LAPP_ACURA_MOM_BACKEND/models/pln"
	"fmt"
	"github.com/go-xorm/xorm"
	"xorm.io/core"
)

/******************************************************************************
 *
 * @Struct Name              : ToyotaCalloffDAOImplement
 *-----------------------------------------------------------------------------
 *
 * @Description              : ToyotaCalloff的数据访问对象实现
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-26 09:51:03
 *
 ******************************************************************************/
type ToyotaCalloffDAOImplement struct {
	session *xorm.Session
	meta    *grmi.Entity
	plantNr int
	userid  string
}

/******************************************************************************
 *
 * @Function Name            : NewToyotaCalloffDAOImplement
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个ToyotaCalloffDAOImplement实例
 *
 * @Function Parameters      : xorm会话
 *
 * @Function Parameters      : 基本主键
 *
 * @Return Value             : ToyotaCalloffDAOImplement实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-26 09:51:03
 *
 ******************************************************************************/
func NewToyotaCalloffDAOImplement(session *xorm.Session, plantNr int, userid string) *ToyotaCalloffDAOImplement {
	return &ToyotaCalloffDAOImplement{session, &meta.ToyotaCalloff, plantNr, userid}
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.InsertOne
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) InsertOne(entity *model.ToyotaCalloff) error {
	entity.PlantNr = impl.plantNr
	entity.LastUser = impl.userid
	_, err := impl.session.Table(impl.meta.TableName).Insert(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.DeleteOne
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) DeleteOne(demandId string) error {
	_, err := impl.session.Table(impl.meta.TableName).ID(core.PK{impl.plantNr, demandId}).Delete(impl.meta.Indicator)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}

	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.SelectOne
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) SelectOne(demandId string) (*model.ToyotaCalloff, error) {
	var data model.ToyotaCalloff
	ok, err := impl.session.Table(impl.meta.TableName).ID(core.PK{impl.plantNr, demandId}).Get(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	if !ok {
		return nil, nil
	}
	return &data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.UpdateOne
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) UpdateOne(entity *model.ToyotaCalloff) error {
	entity.PlantNr = impl.plantNr
	entity.LastUser = impl.userid
	_, err := impl.session.Table(impl.meta.TableName).Cols(impl.meta.UpdatingMembers...).ID(entity.GetKey()).Update(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.Insert
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) Insert(entities *[]model.ToyotaCalloff) error {
	for i := 0; i < len(*entities); i++ {
		entity := &(*entities)[i]
		entity.PlantNr = impl.plantNr
		entity.LastUser = impl.userid
	}
	_, err := impl.session.Table(impl.meta.TableName).Insert(entities)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.Delete
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) Delete(entities *[]model.ToyotaCalloff) error {
	for _, entity := range *entities {
		entity.PlantNr = impl.plantNr
		_, err := impl.session.Table(impl.meta.TableName).ID(entity.GetKey()).Delete(impl.meta.Indicator)
		if err != nil {
			return grmi.NewDataBaseError(err)
		}
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.DeleteWhere
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) DeleteWhere(predicates []grmi.Predicate) error {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.ToyotaCalloff_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	_, err := session.Delete(impl.meta.Indicator)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.Select
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) Select(predicates []grmi.Predicate, orderByFields []grmi.Field) ([]model.ToyotaCalloff, error) {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.ToyotaCalloff_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	session = session.OrderBy(meta.ToyotaCalloff_PlantNr.ColumnName)
	if orderByFields != nil {
		for _, field := range orderByFields {
			session = session.OrderBy(field.ColumnName)
		}
	}

	data := make([]model.ToyotaCalloff, 0, 10)
	err := session.Find(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	return data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.SelectCols
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) SelectCols(predicates []grmi.Predicate, orderByFields []grmi.Field, cloumns []string) ([]map[string]interface{}, error) {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.ToyotaCalloff_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	session = session.OrderBy(meta.ToyotaCalloff_PlantNr.ColumnName)
	if orderByFields != nil {
		for _, field := range orderByFields {
			session = session.OrderBy(field.ColumnName)
		}
	}

	data := make([]map[string]interface{}, 0, 10)
	err := session.Cols(cloumns...).Find(&data)
	if err != nil {
		return nil, grmi.NewDataBaseError(err)
	}
	return data, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.SelectAndPaging
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) SelectAndPaging(paging *grmi.Paging, predicates []grmi.Predicate, orderByFields []grmi.Field) (grmi.PagingResult, error) {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.ToyotaCalloff_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	count, err := session.Count(impl.meta.Indicator)
	if err != nil {
		return grmi.EmptyPagingResult, err
	}

	//orderBy := " order by " + meta.ToyotaCalloff_PlantNr.ColumnName
	//if orderByFields != nil {
	//	for _, field := range orderByFields {
	//		if orderBy == " order by " {
	//			orderBy += field.ColumnName
	//		} else {
	//			orderBy += ", " + field.ColumnName
	//		}
	//	}
	//}
	orderBy := " order by " + meta.ToyotaCalloff_OrderTime.SortColumnName + " desc "
	parameters = append(parameters, paging.Offset(), paging.Size)
	data := make([]model.ToyotaCalloff, 0, 10)
	where = " where " + where
	err = impl.session.Table(impl.meta.TableName).SQL("select * from ["+impl.meta.TableName+"]"+where+orderBy+" offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.EmptyPagingResult, err
	}
	return grmi.PagingResult{Records: data, Count: count, PageNumber: paging.Number, PageSize: paging.Size}, nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.Update
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) Update(entities *[]model.ToyotaCalloff) error {
	for i := 0; i < len(*entities); i++ {
		entity := &(*entities)[i]
		entity.PlantNr = impl.plantNr
		entity.LastUser = impl.userid
		_, err := impl.session.Table(impl.meta.TableName).Cols(impl.meta.UpdatingMembers...).ID(entity.GetKey()).Update(entity)
		if err != nil {
			return grmi.NewDataBaseError(err)
		}
	}
	return nil
}

/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.UpdateWhere
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) UpdateWhere(predicates []grmi.Predicate, entity *model.ToyotaCalloff, columns ...string) error {

	parameters := []interface{}{impl.plantNr}
	where := fmt.Sprintf("%s = ?", meta.ToyotaCalloff_PlantNr.ColumnName)

	session := impl.session.Table(impl.meta.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	columns = append(columns, meta.ToyotaCalloff_LastUser.Name)
	entity.LastUser = impl.userid
	_, err := impl.session.Table(impl.meta.TableName).Cols(columns...).Update(entity)
	if err != nil {
		return grmi.NewDataBaseError(err)
	}
	return nil
}


/******************************************************************************
 *
 * @Reference LAPP_ACURA_MOM_BACKEND/dao/pln/ToyotaCalloffDAO.SelectMaxSeqCallOff
 *
 ******************************************************************************/
func (impl *ToyotaCalloffDAOImplement) SelectMaxSeqCallOff() (*model.ToyotaCalloff, error) {

	parameters := []interface{}{impl.plantNr, model.CALLOFF_STATUS_PARSED}
	where := fmt.Sprintf("%s = ? and %s = ?",
		meta.ToyotaCalloff_PlantNr.ColumnName,
		meta.ToyotaCalloff_Parsed.ColumnName)
	var data model.ToyotaCalloff
	 ok, err := impl.session.Table(impl.meta.TableName).Where(where, parameters...).Desc(meta.ToyotaCalloff_CheckSequence.Name).Get(&data)
	 if err != nil {
		 return nil, err
	 }
	 if !ok {
		 return nil, nil
	 }
	return &data, nil
}

