// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/services/base/implments"
)

/******************************************************************************
 *
 * @Interface Name           : ArticleService
 *-----------------------------------------------------------------------------
 *
 * @Description              : Article的服务接口
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-05-06 15:41:10
 *
 ******************************************************************************/
type ArticleService interface {
	/******************************************************************************
	 *
	 * @Function Name            : InsertOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入一条Article
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的Article
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-03-18 23:20:31
	 *
	 ******************************************************************************/
	InsertOne(*global.User, *model.Article) error
	/******************************************************************************
	 *
	 * @Function Name            : DeleteOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除指定键的Article
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-03-18 23:20:31
	 *
	 ******************************************************************************/
	DeleteOne(*global.User, string, string) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查找指定键的Article
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 查找到的Article
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-03-18 23:20:31
	 *
	 ******************************************************************************/
	SelectOne(*global.User, string, string) (*model.Article, error)
	/******************************************************************************
	 *
	 * @Function Name            : UpdateOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改一条Article
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Article
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-03-18 23:20:31
	 *
	 ******************************************************************************/
	UpdateOne(*global.User, *model.Article) error
	/******************************************************************************
	 *
	 * @Function Name            : Insert
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入多条Article
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的Article列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-03-18 23:20:31
	 *
	 ******************************************************************************/
	Insert(*global.User, *[]model.Article) error
	/******************************************************************************
	 *
	 * @Function Name            : Delete
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除多条Article
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要删除的Article列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-03-18 23:20:31
	 *
	 ******************************************************************************/
	Delete(*global.User, *[]model.Article) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectAndPaging
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询Article
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-03-18 23:20:31
	 *
	 ******************************************************************************/
	Select(*global.User, map[string]string) (interface{}, error)
	/******************************************************************************
	 *
	 * @Function Name            : Update
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改多条Article
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Article列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-03-18 23:20:31
	 *
	 ******************************************************************************/
	Update(*global.User, *[]model.Article) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectByWorkLine
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 通过workLineId查询Article
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : workLineId
	 *
	 * @Return Value             : article list
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : zhangxin
	 *
	 * @Date                     : 2021-10-08
	 *
	 ******************************************************************************/
	SelectByWorkLine(user *global.User, workLineId string) (interface{}, error)
	/******************************************************************************
	 *
	 * @Function Name            : SelectCols
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询Article指定列
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : zhangxin
	 *
	 * @Date                     : 2021-12-22
	 *
	 ******************************************************************************/
	SelectCols(user *global.User, urlParameters map[string]string) (interface{}, error)
	/******************************************************************************
	 *
	 * @Function Name            : ImportFromExcel
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 复制
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 文件路径
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : zhangxin
	 *
	 * @Date                     : 2021-12-22
	 *
	 ******************************************************************************/
	ImportFromExcel(user *global.User, filepath string) error
	/******************************************************************************
	 *
	 * @Function Name            : Copy
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 复制
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 文件路径
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : zhangxin
	 *
	 * @Date                     : 2022-01-04
	 *
	 ******************************************************************************/
	Copy(user *global.User, articleId, projectId string, targetProjectId string) error
}

/******************************************************************************
 *
 * @Function Name            : NewArticleService
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个ArticleService
 *
 * @Return Value             : ArticleService实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-03-18 23:20:31
 *
 ******************************************************************************/
func NewArticleService() ArticleService {
	return implments.NewArticleServiceImplement()
}
