// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/services/base/implments"
)

/******************************************************************************
 *
 * @Interface Name           : CrossPlanTaskService
 *-----------------------------------------------------------------------------
 *
 * @Description              : CrossPlanTask的服务接口
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
type CrossPlanTaskService interface {
	/******************************************************************************
	 *
	 * @Function Name            : InsertOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入一条CrossPlanTask
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的CrossPlanTask
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2022-05-10 22:01:07
	 *
	 ******************************************************************************/
	InsertOne(*global.User, *model.CrossPlanTask) error
	/******************************************************************************
	 *
	 * @Function Name            : DeleteOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除指定键的CrossPlanTask
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2022-05-10 22:01:07
	 *
	 ******************************************************************************/
	DeleteOne(*global.User, int) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查找指定键的CrossPlanTask
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 查找到的CrossPlanTask
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2022-05-10 22:01:07
	 *
	 ******************************************************************************/
	SelectOne(*global.User, int) (*model.CrossPlanTask, error)
	/******************************************************************************
	 *
	 * @Function Name            : UpdateOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改一条CrossPlanTask
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的CrossPlanTask
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2022-05-10 22:01:07
	 *
	 ******************************************************************************/
	UpdateOne(*global.User, *model.CrossPlanTask) error
	/******************************************************************************
	 *
	 * @Function Name            : Insert
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入多条CrossPlanTask
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的CrossPlanTask列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2022-05-10 22:01:07
	 *
	 ******************************************************************************/
	Insert(*global.User, *[]model.CrossPlanTask) error
	/******************************************************************************
	 *
	 * @Function Name            : Delete
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除多条CrossPlanTask
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要删除的CrossPlanTask列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2022-05-10 22:01:07
	 *
	 ******************************************************************************/
	Delete(*global.User, *[]model.CrossPlanTask) error
	/******************************************************************************
	 *
	 * @Function Name            : Select
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询CrossPlanTask
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2022-05-10 22:01:07
	 *
	 ******************************************************************************/
	Select(*global.User, map[string]string) ([]model.CrossPlanTask, error)
	/******************************************************************************
	 *
	 * @Function Name            : SelectAndPaging
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询CrossPlanTask
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2022-05-10 22:01:07
	 *
	 ******************************************************************************/
	SelectAndPaging(*global.User, map[string]string) (grmi.PagingResult, error)
	/******************************************************************************
	 *
	 * @Function Name            : Update
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改多条CrossPlanTask
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的CrossPlanTask列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2022-05-10 22:01:07
	 *
	 ******************************************************************************/
	Update(*global.User, *[]model.CrossPlanTask) error
}

/******************************************************************************
 *
 * @Function Name            : NewCrossPlanTaskService
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个CrossPlanTaskService
 *
 * @Return Value             : CrossPlanTaskService实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func NewCrossPlanTaskService() CrossPlanTaskService {
	return implments.NewCrossPlanTaskServiceImplement()
}
