// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/services/base/implments"
)

/******************************************************************************
 *
 * @Interface Name           : OperationService
 *-----------------------------------------------------------------------------
 *
 * @Description              : Operation的服务接口
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-22 18:49:53
 *
 ******************************************************************************/
type OperationService interface {
	/******************************************************************************
	 *
	 * @Function Name            : InsertOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入一条Operation
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的Operation
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-11-22 18:49:53
	 *
	 ******************************************************************************/
	InsertOne(*global.User, *model.Operation) error
	/******************************************************************************
	 *
	 * @Function Name            : DeleteOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除指定键的Operation
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-11-22 18:49:53
	 *
	 ******************************************************************************/
	DeleteOne(*global.User, int) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查找指定键的Operation
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 查找到的Operation
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-11-22 18:49:53
	 *
	 ******************************************************************************/
	SelectOne(*global.User, int) (*model.Operation, error)
	/******************************************************************************
	 *
	 * @Function Name            : UpdateOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改一条Operation
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Operation
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-11-22 18:49:53
	 *
	 ******************************************************************************/
	UpdateOne(*global.User, *model.Operation) error
	/******************************************************************************
	 *
	 * @Function Name            : Insert
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入多条Operation
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的Operation列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-11-22 18:49:53
	 *
	 ******************************************************************************/
	Insert(*global.User, *[]model.Operation) error
	/******************************************************************************
	 *
	 * @Function Name            : Delete
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除多条Operation
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要删除的Operation列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-11-22 18:49:53
	 *
	 ******************************************************************************/
	Delete(*global.User, *[]model.Operation) error
	/******************************************************************************
	 *
	 * @Function Name            : Select
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询Operation
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-11-22 18:49:53
	 *
	 ******************************************************************************/
	Select(*global.User, map[string]string) ([]model.Operation, error)
	/******************************************************************************
	 *
	 * @Function Name            : SelectAndPaging
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询Operation
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-11-22 18:49:53
	 *
	 ******************************************************************************/
	SelectAndPaging(*global.User, map[string]string) (grmi.PagingResult, error)
	/******************************************************************************
	 *
	 * @Function Name            : Update
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改多条Operation
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Operation列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-11-22 18:49:53
	 *
	 ******************************************************************************/
	Update(*global.User, *[]model.Operation) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectOperationByArtId
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 通过总成id搜索工艺路径
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 总成id
	*
	 * @Function Parameters      : 项目id
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : zhangxin
	 *
	 * @Date                     : 2022-01-05
	 *
	 ******************************************************************************/
	SelectOperationByArtId(user *global.User, productId string, projectId string, workLineId string) (interface{}, error)
	/******************************************************************************
	 *
	 * @Function Name            : Copy
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 通过项目复制工艺
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 来源项目id
	*
	 * @Function Parameters      : 目标项目id
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : zhangxin
	 *
	 * @Date                     : 2022-01-24
	 *
	 ******************************************************************************/
	Copy(user *global.User, po string, sourceProjectId string, targetProjectId string, source string) (err error)
}

/******************************************************************************
 *
 * @Function Name            : NewOperationService
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个OperationService
 *
 * @Return Value             : OperationService实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-22 18:49:53
 *
 ******************************************************************************/
func NewOperationService() OperationService {
	return implments.NewOperationServiceImplement()
}
