// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/services/base/implments"
)

/******************************************************************************
 *
 * @Interface Name           : PersonService
 *-----------------------------------------------------------------------------
 *
 * @Description              : Person的服务接口
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-25 14:20:56
 *
 ******************************************************************************/
type PersonService interface {
	/******************************************************************************
	 *
	 * @Function Name            : InsertOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入一条Person
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的Person
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-04-25 14:20:56
	 *
	 ******************************************************************************/
	InsertOne(*global.User, *model.Person) error
	/******************************************************************************
	 *
	 * @Function Name            : DeleteOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除指定键的Person
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-04-25 14:20:56
	 *
	 ******************************************************************************/
	DeleteOne(*global.User, int) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查找指定键的Person
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 查找到的Person
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-04-25 14:20:56
	 *
	 ******************************************************************************/
	SelectOne(*global.User, int) (*model.Person, error)
	/******************************************************************************
	 *
	 * @Function Name            : UpdateOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改一条Person
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Person
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-04-25 14:20:56
	 *
	 ******************************************************************************/
	UpdateOne(*global.User, *model.Person) error
	/******************************************************************************
	 *
	 * @Function Name            : Insert
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入多条Person
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的Person列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-04-25 14:20:56
	 *
	 ******************************************************************************/
	Insert(*global.User, *[]model.Person) error
	/******************************************************************************
	 *
	 * @Function Name            : Delete
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除多条Person
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要删除的Person列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-04-25 14:20:56
	 *
	 ******************************************************************************/
	Delete(*global.User, *[]model.Person) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectAndPaging
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询Person
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-04-25 14:20:56
	 *
	 ******************************************************************************/
	Select(*global.User, map[string]string) (interface{}, error)
	/******************************************************************************
	 *
	 * @Function Name            : Update
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改多条Person
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Person列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-04-25 14:20:56
	 *
	 ******************************************************************************/
	Update(*global.User, *[]model.Person) error
	/******************************************************************************
	 *
	 * @Function Name            : CheckLogin
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 人员登录
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Person列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-04-25 14:20:56
	 *
	 ******************************************************************************/
	CheckLogin(*global.User, string, string) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectOneByUserName
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查找指定键的Person
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 查找到的Person
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-04-25 14:20:56
	 *
	 ******************************************************************************/
	SelectOneByUserName(*global.User, string) (*model.Person, error)
}

/******************************************************************************
 *
 * @Function Name            : NewPersonService
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个PersonService
 *
 * @Return Value             : PersonService实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-25 14:20:56
 *
 ******************************************************************************/
func NewPersonService() PersonService {
	return implments.NewPersonServiceImplement()
}
