// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/services/base/implments"
)

type (
	/******************************************************************************
	 *
	 * @Interface Name           : ProcessService
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : Process的服务接口
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-11-22 17:33:27
	 *
	 ******************************************************************************/
	ProcessService interface {
		/******************************************************************************
		 *
		 * @Function Name            : InsertOne
		 *-----------------------------------------------------------------------------
		 *
		 * @Description              : 插入一条Process
		 *
		 * @Function Parameters      : 当前访问人员信息
		 *
		 * @Function Parameters      : 需要插入的Process
		 *
		 * @Return Value             : 执行时发生的错误
		 *
		 * @Author                   : 代码生成器创建
		 *
		 * @Date                     : 2021-11-22 17:33:27
		 *
		 ******************************************************************************/
		InsertOne(*global.User, *model.Process) error
		/******************************************************************************
		 *
		 * @Function Name            : DeleteOne
		 *-----------------------------------------------------------------------------
		 *
		 * @Description              : 删除指定键的Process
		 *
		 * @Function Parameters      : 主键
		 *
		 * @Return Value             : 执行时发生的错误
		 *
		 * @Author                   : 代码生成器创建
		 *
		 * @Date                     : 2021-11-22 17:33:27
		 *
		 ******************************************************************************/
		DeleteOne(*global.User, string) error
		/******************************************************************************
		 *
		 * @Function Name            : SelectOne
		 *-----------------------------------------------------------------------------
		 *
		 * @Description              : 查找指定键的Process
		 *
		 * @Function Parameters      : 主键
		 *
		 * @Return Value             : 查找到的Process
		 *
		 * @Return Value             : 执行时发生的错误
		 *
		 * @Author                   : 代码生成器创建
		 *
		 * @Date                     : 2021-11-22 17:33:27
		 *
		 ******************************************************************************/
		SelectOne(*global.User, string) (*model.Process, error)
		/******************************************************************************
		 *
		 * @Function Name            : UpdateOne
		 *-----------------------------------------------------------------------------
		 *
		 * @Description              : 修改一条Process
		 *
		 * @Function Parameters      : 当前访问人员信息
		 *
		 * @Function Parameters      : 需要修改的Process
		 *
		 * @Return Value             : 执行时发生的错误
		 *
		 * @Author                   : 代码生成器创建
		 *
		 * @Date                     : 2021-11-22 17:33:27
		 *
		 ******************************************************************************/
		UpdateOne(*global.User, *model.Process) error
		/******************************************************************************
		 *
		 * @Function Name            : Insert
		 *-----------------------------------------------------------------------------
		 *
		 * @Description              : 插入多条Process
		 *
		 * @Function Parameters      : 当前访问人员信息
		 *
		 * @Function Parameters      : 需要插入的Process列表
		 *
		 * @Return Value             : 执行时发生的错误
		 *
		 * @Author                   : 代码生成器创建
		 *
		 * @Date                     : 2021-11-22 17:33:27
		 *
		 ******************************************************************************/
		Insert(*global.User, *[]model.Process) error
		/******************************************************************************
		 *
		 * @Function Name            : Delete
		 *-----------------------------------------------------------------------------
		 *
		 * @Description              : 删除多条Process
		 *
		 * @Function Parameters      : 当前访问人员信息
		 *
		 * @Function Parameters      : 需要删除的Process列表
		 *
		 * @Return Value             : 执行时发生的错误
		 *
		 * @Author                   : 代码生成器创建
		 *
		 * @Date                     : 2021-11-22 17:33:27
		 *
		 ******************************************************************************/
		Delete(*global.User, *[]model.Process) error
		/******************************************************************************
		 *
		 * @Function Name            : Select
		 *-----------------------------------------------------------------------------
		 *
		 * @Description              : 查询Process
		 *
		 * @Function Parameters      : 当前访问人员信息
		 *
		 * @Function Parameters      : 查询参数
		 *
		 * @Return Value             : 查询结果
		 *
		 * @Return Value             : 执行时发生的错误
		 *
		 * @Author                   : 代码生成器创建
		 *
		 * @Date                     : 2021-11-22 17:33:27
		 *
		 ******************************************************************************/
		Select(*global.User, map[string]string) ([]model.Process, error)
		/******************************************************************************
		 *
		 * @Function Name            : SelectAndPaging
		 *-----------------------------------------------------------------------------
		 *
		 * @Description              : 查询Process
		 *
		 * @Function Parameters      : 当前访问人员信息
		 *
		 * @Function Parameters      : 查询参数
		 *
		 * @Return Value             : 查询结果
		 *
		 * @Return Value             : 执行时发生的错误
		 *
		 * @Author                   : 代码生成器创建
		 *
		 * @Date                     : 2021-11-22 17:33:27
		 *
		 ******************************************************************************/
		SelectAndPaging(*global.User, map[string]string) (grmi.PagingResult, error)
		/******************************************************************************
		 *
		 * @Function Name            : Update
		 *-----------------------------------------------------------------------------
		 *
		 * @Description              : 修改多条Process
		 *
		 * @Function Parameters      : 当前访问人员信息
		 *
		 * @Function Parameters      : 需要修改的Process列表
		 *
		 * @Return Value             : 执行时发生的错误
		 *
		 * @Author                   : 代码生成器创建
		 *
		 * @Date                     : 2021-11-22 17:33:27
		 *
		 ******************************************************************************/
		Update(*global.User, *[]model.Process) error
		/******************************************************************************
		 *
		 * @Function Name            : SelectByWorkLine
		 *-----------------------------------------------------------------------------
		 *
		 * @Description              : 通过workline查询工序
		 *
		 * @Function Parameters      : 当前访问人员信息
		 *
		 * @Function Parameters      : work line id
		*
		 * @Return Value             : 工序列表
		 *
		 * @Return Value             : 执行时发生的错误
		 *
		 * @Author                   : zhangxin
		 *
		 * @Date                     : 2021-11-23
		 *
		 ******************************************************************************/
		SelectByWorkLine(user *global.User, workLineId string) ([]model.Process, error)
		/******************************************************************************
		 *
		 * @Function Name            : SelectOperation
		 *-----------------------------------------------------------------------------
		 *
		 * @Description              : 通过工序查询工步
		 *
		 * @Function Parameters      : 当前访问人员信息
		 *
		 * @Function Parameters      : 工序id
		*
		 * @Return Value             : 工步树形结构
		 *
		 * @Return Value             : 执行时发生的错误
		 *
		 * @Author                   : zhangxin
		 *
		 * @Date                     : 2021-11-23
		 *
		 ******************************************************************************/
		SelectOperation(user *global.User, po string) ([]model.OperationDataTree, error)
		/******************************************************************************
		 *
		 * @Function Name            : SelectMonitorOperation
		 *-----------------------------------------------------------------------------
		 *
		 * @Description              : 通过工序查询监控工步
		 *
		 * @Function Parameters      : 当前访问人员信息
		 *
		 * @Function Parameters      : 工序id
		*
		 * @Return Value             : 工步树形结构
		 *
		 * @Return Value             : 执行时发生的错误
		 *
		 * @Author                   : zhangxin
		 *
		 * @Date                     : 2021-11-23
		 *
		 ******************************************************************************/
		SelectMonitorOperation(user *global.User, po string) ([]model.OperationDataTree, error)
	}
)

/******************************************************************************
 *
 * @Function Name            : NewProcessService
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个ProcessService
 *
 * @Return Value             : ProcessService实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-22 17:33:27
 *
 ******************************************************************************/
func NewProcessService() ProcessService {
	return implments.NewProcessServiceImplement()
}
