// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/services/base/implments"
)

/******************************************************************************
 *
 * @Interface Name           : SnrService
 *-----------------------------------------------------------------------------
 *
 * @Description              : Snr的服务接口
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
type SnrService interface {
	/******************************************************************************
	 *
	 * @Function Name            : InsertOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入一条Snr
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的Snr
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-08-20 11:11:05
	 *
	 ******************************************************************************/
	InsertOne(*global.User, *model.Snr) error
	/******************************************************************************
	 *
	 * @Function Name            : DeleteOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除指定键的Snr
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-08-20 11:11:05
	 *
	 ******************************************************************************/
	DeleteOne(*global.User, string) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查找指定键的Snr
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 查找到的Snr
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-08-20 11:11:05
	 *
	 ******************************************************************************/
	SelectOne(*global.User, string) (*model.Snr, error)
	/******************************************************************************
	 *
	 * @Function Name            : UpdateOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改一条Snr
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Snr
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-08-20 11:11:05
	 *
	 ******************************************************************************/
	UpdateOne(*global.User, *model.Snr) error
	/******************************************************************************
	 *
	 * @Function Name            : Insert
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入多条Snr
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的Snr列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-08-20 11:11:05
	 *
	 ******************************************************************************/
	Insert(*global.User, *[]model.Snr) error
	/******************************************************************************
	 *
	 * @Function Name            : Delete
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除多条Snr
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要删除的Snr列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-08-20 11:11:05
	 *
	 ******************************************************************************/
	Delete(*global.User, *[]model.Snr) error
	/******************************************************************************
	 *
	 * @Function Name            : Select
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询Snr
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-08-20 11:11:05
	 *
	 ******************************************************************************/
	Select(*global.User, map[string]string) ([]model.Snr, error)
	/******************************************************************************
	 *
	 * @Function Name            : SelectAndPaging
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询Snr
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-08-20 11:11:05
	 *
	 ******************************************************************************/
	SelectAndPaging(*global.User, map[string]string) (grmi.PagingResult, error)
	/******************************************************************************
	 *
	 * @Function Name            : Update
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改多条Snr
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Snr列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-08-20 11:11:05
	 *
	 ******************************************************************************/
	Update(*global.User, *[]model.Snr) error
}

/******************************************************************************
 *
 * @Function Name            : NewSnrService
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个SnrService
 *
 * @Return Value             : SnrService实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func NewSnrService() SnrService {
	return implments.NewSnrServiceImplement()
}
