// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/services/base/implments"
	"LAPP_ACURA_MOM_BACKEND/services/schedule"
	"time"
)

/******************************************************************************
 *
 * @Interface Name           : WorkLineService
 *-----------------------------------------------------------------------------
 *
 * @Description              : WorkLine的服务接口
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-28 14:57:09
 *
 ******************************************************************************/
type WorkLineService interface {
	/******************************************************************************
	 *
	 * @Function Name            : InsertOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入一条WorkLine
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的WorkLine
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-28 14:57:09
	 *
	 ******************************************************************************/
	InsertOne(*global.User, *model.WorkLine) error
	/******************************************************************************
	 *
	 * @Function Name            : DeleteOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除指定键的WorkLine
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-28 14:57:09
	 *
	 ******************************************************************************/
	DeleteOne(*global.User, string) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查找指定键的WorkLine
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 查找到的WorkLine
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-28 14:57:09
	 *
	 ******************************************************************************/
	SelectOne(*global.User, string) (*model.WorkLine, error)
	/******************************************************************************
	 *
	 * @Function Name            : UpdateOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改一条WorkLine
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的WorkLine
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-28 14:57:09
	 *
	 ******************************************************************************/
	UpdateOne(*global.User, *model.WorkLine) error
	/******************************************************************************
	 *
	 * @Function Name            : Insert
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入多条WorkLine
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的WorkLine列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-28 14:57:09
	 *
	 ******************************************************************************/
	Insert(*global.User, *[]model.WorkLine) error
	/******************************************************************************
	 *
	 * @Function Name            : Delete
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除多条WorkLine
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要删除的WorkLine列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-03-18 23:20:31
	 *
	 ******************************************************************************/
	Delete(*global.User, *[]model.WorkLine) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectAndPaging
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询WorkLine
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-03-18 23:20:31
	 *
	 ******************************************************************************/
	Select(*global.User, map[string]string) (interface{}, error)
	/******************************************************************************
	 *
	 * @Function Name            : Update
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改多条WorkLine
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的WorkLine列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-03-18 23:20:31
	 *
	 ******************************************************************************/
	Update(*global.User, *[]model.WorkLine) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectJoinWorkPlace
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 联查WorkLine和工位的关系
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-03-18 23:20:31
	 *
	 ******************************************************************************/
	SelectJoinWorkPlace(*global.User) ([]model.WorkLine, error)
	/******************************************************************************
	 *
	 * @Function Name            : WorkLineTree
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查出所有的Workline
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 娄文智
	 *
	 * @Date                     : 2021-03-24 09:53:49
	 *
	 ******************************************************************************/
	WorkLineTree(user *global.User) (model.WorkLineArr, error)
	/******************************************************************************
	 *
	 * @Function Name            : WorkLineScheduler
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 生产调度排程（一键排程）
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 娄文智
	 *
	 * @Date                     : 2021-03-24 09:53:49
	 *
	 ******************************************************************************/
	WorkLineScheduler(user *global.User, worklinetype string, startDate time.Time, endDate time.Time) (schedule.SchedulerSrv, error)
	/******************************************************************************
	 *
	 * @Function Name            : WorkLineRepairSchedule
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 生产调度排程（人工调度）
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 娄文智
	 *
	 * @Date                     : 2021-03-24 09:53:49
	 *
	 ******************************************************************************/
	WorkLineRepairSchedule(user *global.User, data schedule.SchedulerSrv) (schedule.SchedulerSrv, error)
	/******************************************************************************
	 *
	 * @Function Name            : WorkLineSchedulerManPower
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 生产调度排程（人工调度）
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 娄文智
	 *
	 * @Date                     : 2021-03-24 09:53:49
	 *
	 ******************************************************************************/
	WorkLineSchedulerManPower(user *global.User, worklinetype string, startDate time.Time, endDate time.Time) (schedule.SchedulerSrv, error)
	/******************************************************************************
	 *
	 * @Function Name            : WorkLineSchedulerManualr
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 生产调度排程（人工调度）
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 娄文智
	 *
	 * @Date                     : 2021-03-24 09:53:49
	 *
	 ******************************************************************************/
	WorkLineSchedulerManualr(user *global.User, data schedule.SchedulerSrv) (schedule.SchedulerSrv, error)
	/******************************************************************************
	 *
	 * @Function Name            : SaveWorkLineScheduler
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 保存排程结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 娄文智
	 *
	 * @Date                     : 2021-03-24 09:53:49
	 *
	 ******************************************************************************/
	SaveWorkLineScheduler(user *global.User, data schedule.SchedulerData) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectVirtualWorkLine
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询虚拟产线
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Return Value             : 产线列表
	*
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : zhangxin
	 *
	 * @Date                     : 2021-04-19
	 *
	 ******************************************************************************/
	SelectVirtualWorkLine(user *global.User) ([]model.WorkLine, error)
}

/******************************************************************************
 *
 * @Function Name            : NewWorkLineService
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个WorkLineService
 *
 * @Return Value             : WorkLineService实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-28 14:57:09
 *
 ******************************************************************************/
func NewWorkLineService() WorkLineService {
	return implments.NewWorkLineServiceImplement()
}
