// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package me

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	baseModel "LAPP_ACURA_MOM_BACKEND/models/base"
	model "LAPP_ACURA_MOM_BACKEND/models/me"
	"LAPP_ACURA_MOM_BACKEND/services/me/implments"
)

/******************************************************************************
 *
 * @Interface Name           : ProductService
 *-----------------------------------------------------------------------------
 *
 * @Description              : Product的服务接口
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 15:22:21
 *
 ******************************************************************************/
type ProductService interface {
	/******************************************************************************
	 *
	 * @Function Name            : InsertOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入一条Product
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的Product
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-19 15:22:21
	 *
	 ******************************************************************************/
	InsertOne(*global.User, *model.Product) error
	/******************************************************************************
	 *
	 * @Function Name            : DeleteOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除指定键的Product
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-19 15:22:21
	 *
	 ******************************************************************************/
	DeleteOne(*global.User, string, string, string) error
	/******************************************************************************
	 *
	 * @Function Name            : Cancel
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除指定键的Product
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-19 15:22:21
	 *
	 ******************************************************************************/
	Cancel(*global.User, string, string, string) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查找指定键的Product
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 查找到的Product
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-19 15:22:21
	 *
	 ******************************************************************************/
	SelectOne(*global.User, string, string) (*model.Product, error)
	/******************************************************************************
	 *
	 * @Function Name            : UpdateOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改一条Product
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Product
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-19 15:22:21
	 *
	 ******************************************************************************/
	UpdateOne(*global.User, *model.Product) error
	/******************************************************************************
	 *
	 * @Function Name            : Insert
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入多条Product
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的Product列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-19 15:22:21
	 *
	 ******************************************************************************/
	Insert(*global.User, *[]model.Product) error
	/******************************************************************************
	 *
	 * @Function Name            : Delete
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除多条Product
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要删除的Product列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-19 15:22:21
	 *
	 ******************************************************************************/
	Delete(*global.User, *[]model.Product) error
	/******************************************************************************
	 *
	 * @Function Name            : Select
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询Product
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-19 15:22:21
	 *
	 ******************************************************************************/
	Select(*global.User, map[string]string) ([]model.Product, error)
	/******************************************************************************
	 *
	 * @Function Name            : SelectAndPaging
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询Product
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-19 15:22:21
	 *
	 ******************************************************************************/
	SelectAndPaging(*global.User, map[string]string) (grmi.PagingResult, error)
	/******************************************************************************
	 *
	 * @Function Name            : Update
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改多条Product
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Product列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-10-19 15:22:21
	 *
	 ******************************************************************************/
	Update(*global.User, *[]model.Product) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectByWorkLine
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 通过产线查询可以生产的所有产品
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : worklineid
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : zhangxin
	 *
	 * @Date                     : 2021-11-15
	 *
	 ******************************************************************************/
	SelectByWorkLine(user *global.User, workLineId string) ([]model.Product, error)
	/******************************************************************************
	 *
	 * @Function Name            : Copy
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 复制
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : zhangxin
	 *
	 * @Date                     : 2022-01-04
	 *
	 ******************************************************************************/
	Copy(user *global.User, productId, projectId string, targetProjectId string) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectWorkLineByProduct
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 通过总成查询可生产产线
	 *
	 * @Function Parameters      : 当前访问人员信息
		 *
	 * @Function Parameters      : productId
		 *
	 * @Function Parameters      : projectId
		 *
	 * @Return Value             : 产线列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : zhangxin
	 *
	 * @Date                     : 2022-04-18
	 *
	 ******************************************************************************/
	SelectWorkLineByProduct(user *global.User, productId, projectId string) ([]baseModel.WorkLine, error)
}

/******************************************************************************
 *
 * @Function Name            : NewProductService
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个ProductService
 *
 * @Return Value             : ProductService实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 15:22:21
 *
 ******************************************************************************/
func NewProductService() ProductService {
	return implments.NewProductServiceImplement()
}
