// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package ods

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	model "LAPP_ACURA_MOM_BACKEND/models/ods"
	"LAPP_ACURA_MOM_BACKEND/services/ods/implments"
)

/******************************************************************************
 *
 * @Interface Name           : StationOrderService
 *-----------------------------------------------------------------------------
 *
 * @Description              : StationOrder的服务接口
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-18 11:12:43
 *
 ******************************************************************************/
type StationOrderService interface {
	/******************************************************************************
	 *
	 * @Function Name            : InsertOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入一条StationOrder
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的StationOrder
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-06-18 11:12:43
	 *
	 ******************************************************************************/
	InsertOne(*global.User, *model.StationOrder) error
	/******************************************************************************
	 *
	 * @Function Name            : DeleteOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除指定键的StationOrder
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-06-18 11:12:43
	 *
	 ******************************************************************************/
	DeleteOne(*global.User, string) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查找指定键的StationOrder
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 查找到的StationOrder
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-06-18 11:12:43
	 *
	 ******************************************************************************/
	SelectOne(*global.User, string) (*model.StationOrder, error)
	/******************************************************************************
	 *
	 * @Function Name            : UpdateOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改一条StationOrder
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的StationOrder
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-06-18 11:12:43
	 *
	 ******************************************************************************/
	UpdateOne(*global.User, *model.StationOrder) error
	/******************************************************************************
	 *
	 * @Function Name            : Insert
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入多条StationOrder
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的StationOrder列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-06-18 11:12:43
	 *
	 ******************************************************************************/
	Insert(*global.User, *[]model.StationOrder) error
	/******************************************************************************
	 *
	 * @Function Name            : Delete
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除多条StationOrder
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要删除的StationOrder列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-06-18 11:12:43
	 *
	 ******************************************************************************/
	Delete(*global.User, *[]model.StationOrder) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectAndPaging
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询StationOrder
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-06-18 11:12:43
	 *
	 ******************************************************************************/
	Select(*global.User, map[string]string) (interface{}, error)
	/******************************************************************************
	 *
	 * @Function Name            : Update
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改多条StationOrder
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的StationOrder列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-06-18 11:12:43
	 *
	 ******************************************************************************/
	Update(*global.User, *[]model.StationOrder) error
}

/******************************************************************************
 *
 * @Function Name            : NewStationOrderService
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个StationOrderService
 *
 * @Return Value             : StationOrderService实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-18 11:12:43
 *
 ******************************************************************************/
func NewStationOrderService() StationOrderService {
	return implments.NewStationOrderServiceImplement()
}
