// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package wm

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	model "LAPP_ACURA_MOM_BACKEND/models/wm"
	"LAPP_ACURA_MOM_BACKEND/services/wm/implments"
)

/******************************************************************************
 *
 * @Interface Name           : StorageBinService
 *-----------------------------------------------------------------------------
 *
 * @Description              : StorageBin的服务接口
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-08 11:25:15
 *
 ******************************************************************************/
type StorageBinService interface {
	/******************************************************************************
	 *
	 * @Function Name            : InsertOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入一条StorageBin
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的StorageBin
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-08 11:25:15
	 *
	 ******************************************************************************/
	InsertOne(*global.User, *model.StorageBin) error
	/******************************************************************************
	 *
	 * @Function Name            : DeleteOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除指定键的StorageBin
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-08 11:25:15
	 *
	 ******************************************************************************/
	DeleteOne(*global.User, string, string) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查找指定键的StorageBin
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 查找到的StorageBin
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-08 11:25:15
	 *
	 ******************************************************************************/
	SelectOne(*global.User, string, string) (*model.StorageBin, error)
	/******************************************************************************
	 *
	 * @Function Name            : UpdateOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改一条StorageBin
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的StorageBin
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-08 11:25:15
	 *
	 ******************************************************************************/
	UpdateOne(*global.User, *model.StorageBin) error
	/******************************************************************************
	 *
	 * @Function Name            : Insert
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入多条StorageBin
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的StorageBin列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-08 11:25:15
	 *
	 ******************************************************************************/
	Insert(*global.User, *[]model.StorageBin) error
	/******************************************************************************
	 *
	 * @Function Name            : Delete
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除多条StorageBin
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要删除的StorageBin列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-08 11:25:15
	 *
	 ******************************************************************************/
	Delete(*global.User, *[]model.StorageBin) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectAndPaging
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询StorageBin
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-08 11:25:15
	 *
	 ******************************************************************************/
	Select(*global.User, map[string]string) (interface{}, error)
	/******************************************************************************
	 *
	 * @Function Name            : Update
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改多条StorageBin
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的StorageBin列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-08 11:25:15
	 *
	 ******************************************************************************/
	Update(*global.User, *[]model.StorageBin) error
}

/******************************************************************************
 *
 * @Function Name            : NewStorageBinService
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个StorageBinService
 *
 * @Return Value             : StorageBinService实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-08 11:25:15
 *
 ******************************************************************************/
func NewStorageBinService() StorageBinService {
	return implments.NewStorageBinServiceImplement()
}
