// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package wm

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	model "LAPP_ACURA_MOM_BACKEND/models/wm"
	"LAPP_ACURA_MOM_BACKEND/services/wm/implments"
)

/******************************************************************************
 *
 * @Interface Name           : VendorService
 *-----------------------------------------------------------------------------
 *
 * @Description              : Vendor的服务接口
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-21 11:04:46
 *
 ******************************************************************************/
type VendorService interface {
	/******************************************************************************
	 *
	 * @Function Name            : InsertOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入一条Vendor
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的Vendor
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-21 11:04:46
	 *
	 ******************************************************************************/
	InsertOne(*global.User, *model.Vendor) error
	/******************************************************************************
	 *
	 * @Function Name            : DeleteOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除指定键的Vendor
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-21 11:04:46
	 *
	 ******************************************************************************/
	DeleteOne(*global.User, string) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查找指定键的Vendor
	 *
	 * @Function Parameters      : 主键
	 *
	 * @Return Value             : 查找到的Vendor
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-21 11:04:46
	 *
	 ******************************************************************************/
	SelectOne(*global.User, string) (*model.Vendor, error)
	/******************************************************************************
	 *
	 * @Function Name            : UpdateOne
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改一条Vendor
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Vendor
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-21 11:04:46
	 *
	 ******************************************************************************/
	UpdateOne(*global.User, *model.Vendor) error
	/******************************************************************************
	 *
	 * @Function Name            : Insert
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 插入多条Vendor
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要插入的Vendor列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-21 11:04:46
	 *
	 ******************************************************************************/
	Insert(*global.User, *[]model.Vendor) error
	/******************************************************************************
	 *
	 * @Function Name            : Delete
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 删除多条Vendor
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要删除的Vendor列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-21 11:04:46
	 *
	 ******************************************************************************/
	Delete(*global.User, *[]model.Vendor) error
	/******************************************************************************
	 *
	 * @Function Name            : SelectAndPaging
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 查询Vendor
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 查询参数
	 *
	 * @Return Value             : 查询结果
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-21 11:04:46
	 *
	 ******************************************************************************/
	Select(*global.User, map[string]string) (interface{}, error)
	/******************************************************************************
	 *
	 * @Function Name            : Update
	 *-----------------------------------------------------------------------------
	 *
	 * @Description              : 修改多条Vendor
	 *
	 * @Function Parameters      : 当前访问人员信息
	 *
	 * @Function Parameters      : 需要修改的Vendor列表
	 *
	 * @Return Value             : 执行时发生的错误
	 *
	 * @Author                   : 代码生成器创建
	 *
	 * @Date                     : 2021-07-21 11:04:46
	 *
	 ******************************************************************************/
	Update(*global.User, *[]model.Vendor) error
}

/******************************************************************************
 *
 * @Function Name            : NewVendorService
 *-----------------------------------------------------------------------------
 *
 * @Description              : 创建一个VendorService
 *
 * @Return Value             : VendorService实例
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-21 11:04:46
 *
 ******************************************************************************/
func NewVendorService() VendorService {
	return implments.NewVendorServiceImplement()
}
