// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneCrossPlanTask
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条CrossPlanTask的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func RegisterOneCrossPlanTask(party router.Party, httpMethod string, path string, method func(*global.User, *model.CrossPlanTask) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil

		entity := new(model.CrossPlanTask)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfCrossPlanTask
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条CrossPlanTask的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func RegisterIDOfCrossPlanTask(party router.Party, httpMethod string, path string, method func(*global.User, int) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		taskNr, err := ctx.Params().GetInt("taskNr")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, taskNr)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneCrossPlanTask
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条CrossPlanTask的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func RegisterInsertOneCrossPlanTask(party router.Party, path string, method func(*global.User, *model.CrossPlanTask) error) {

	RegisterOneCrossPlanTask(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneCrossPlanTask
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条CrossPlanTask的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func RegisterDeleteOneCrossPlanTask(party router.Party, path string, method func(*global.User, int) error) {

	RegisterIDOfCrossPlanTask(party, "DELETE", path+"/{taskNr:int}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneCrossPlanTask
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条CrossPlanTask的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func RegisterSelectOneCrossPlanTask(party router.Party, path string, method func(*global.User, int) (*model.CrossPlanTask, error)) {

	party.Get(path+"/{taskNr:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		taskNr, err := ctx.Params().GetInt("taskNr")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		result, err := method(user, taskNr)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneCrossPlanTask
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条CrossPlanTask的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func RegisterUpdateOneCrossPlanTask(party router.Party, path string, method func(*global.User, *model.CrossPlanTask) error) {

	RegisterOneCrossPlanTask(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiCrossPlanTask
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条CrossPlanTask的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func RegisterMultiCrossPlanTask(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.CrossPlanTask) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		entities := make([]model.CrossPlanTask, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertCrossPlanTask
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条CrossPlanTask的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func RegisterInsertCrossPlanTask(party router.Party, path string, method func(*global.User, *[]model.CrossPlanTask) error) {

	RegisterMultiCrossPlanTask(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteCrossPlanTask
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条CrossPlanTask的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func RegisterDeleteCrossPlanTask(party router.Party, path string, method func(*global.User, *[]model.CrossPlanTask) error) {

	RegisterMultiCrossPlanTask(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectCrossPlanTask
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询CrossPlanTask的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func RegisterSelectCrossPlanTask(party router.Party, path string, method func(*global.User, map[string]string) ([]model.CrossPlanTask, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryCrossPlanTask
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询CrossPlanTask并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func RegisterQueryCrossPlanTask(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateCrossPlanTask
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条CrossPlanTask的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-05-10 22:01:07
 *
 ******************************************************************************/
func RegisterUpdateCrossPlanTask(party router.Party, path string, method func(*global.User, *[]model.CrossPlanTask) error) {

	RegisterMultiCrossPlanTask(party, "PUT", path, method)
}
