// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneLabelParserHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条LabelParserHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterOneLabelParserHead(party router.Party, httpMethod string, path string, method func(*global.User, *model.LabelParserHead) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.LabelParserHead)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfLabelParserHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条LabelParserHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterIDOfLabelParserHead(party router.Party, httpMethod string, path string, method func(*global.User, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		parserId := ctx.Params().GetString("parserId")

		err = method(user, parserId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneLabelParserHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条LabelParserHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterInsertOneLabelParserHead(party router.Party, path string, method func(*global.User, *model.LabelParserHead) error) {

	RegisterOneLabelParserHead(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneLabelParserHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条LabelParserHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterDeleteOneLabelParserHead(party router.Party, path string, method func(*global.User, string) error) {

	RegisterIDOfLabelParserHead(party, "DELETE", path+"/{parserId:string}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneLabelParserHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条LabelParserHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterSelectOneLabelParserHead(party router.Party, path string, method func(*global.User, string) (*model.LabelParserHead, error)) {

	party.Get(path+"/{parserId:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		parserId := ctx.Params().GetString("parserId")
		result, err := method(user, parserId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneLabelParserHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条LabelParserHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterUpdateOneLabelParserHead(party router.Party, path string, method func(*global.User, *model.LabelParserHead) error) {

	RegisterOneLabelParserHead(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiLabelParserHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条LabelParserHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterMultiLabelParserHead(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.LabelParserHead) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.LabelParserHead, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertLabelParserHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条LabelParserHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterInsertLabelParserHead(party router.Party, path string, method func(*global.User, *[]model.LabelParserHead) error) {

	RegisterMultiLabelParserHead(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteLabelParserHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条LabelParserHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterDeleteLabelParserHead(party router.Party, path string, method func(*global.User, *[]model.LabelParserHead) error) {

	RegisterMultiLabelParserHead(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectLabelParserHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询LabelParserHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterSelectLabelParserHead(party router.Party, path string, method func(*global.User, map[string]string) ([]model.LabelParserHead, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryLabelParserHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询LabelParserHead并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterQueryLabelParserHead(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateLabelParserHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条LabelParserHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterUpdateLabelParserHead(party router.Party, path string, method func(*global.User, *[]model.LabelParserHead) error) {

	RegisterMultiLabelParserHead(party, "PUT", path, method)
}
