// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneMonitorOperation
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条MonitorOperation的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-25 09:48:01
 *
 ******************************************************************************/
func RegisterOneMonitorOperation(party router.Party, httpMethod string, path string, method func(*global.User, *model.MonitorOperation) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil

		entity := new(model.MonitorOperation)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfMonitorOperation
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条MonitorOperation的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-25 09:48:01
 *
 ******************************************************************************/
func RegisterIDOfMonitorOperation(party router.Party, httpMethod string, path string, method func(*global.User, int) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		operationId, err := ctx.Params().GetInt("operationId")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, operationId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneMonitorOperation
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条MonitorOperation的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-25 09:48:01
 *
 ******************************************************************************/
func RegisterInsertOneMonitorOperation(party router.Party, path string, method func(*global.User, *model.MonitorOperation) error) {

	RegisterOneMonitorOperation(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneMonitorOperation
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条MonitorOperation的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-25 09:48:01
 *
 ******************************************************************************/
func RegisterDeleteOneMonitorOperation(party router.Party, path string, method func(*global.User, int) error) {

	RegisterIDOfMonitorOperation(party, "DELETE", path+"/{operationId:int}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneMonitorOperation
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条MonitorOperation的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-25 09:48:01
 *
 ******************************************************************************/
func RegisterSelectOneMonitorOperation(party router.Party, path string, method func(*global.User, int) (*model.MonitorOperation, error)) {

	party.Get(path+"/{operationId:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		operationId, err := ctx.Params().GetInt("operationId")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		result, err := method(user, operationId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneMonitorOperation
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条MonitorOperation的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-25 09:48:01
 *
 ******************************************************************************/
func RegisterUpdateOneMonitorOperation(party router.Party, path string, method func(*global.User, *model.MonitorOperation) error) {

	RegisterOneMonitorOperation(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiMonitorOperation
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条MonitorOperation的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-25 09:48:01
 *
 ******************************************************************************/
func RegisterMultiMonitorOperation(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.MonitorOperation) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		entities := make([]model.MonitorOperation, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertMonitorOperation
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条MonitorOperation的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-25 09:48:01
 *
 ******************************************************************************/
func RegisterInsertMonitorOperation(party router.Party, path string, method func(*global.User, *[]model.MonitorOperation) error) {

	RegisterMultiMonitorOperation(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteMonitorOperation
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条MonitorOperation的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-25 09:48:01
 *
 ******************************************************************************/
func RegisterDeleteMonitorOperation(party router.Party, path string, method func(*global.User, *[]model.MonitorOperation) error) {

	RegisterMultiMonitorOperation(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectMonitorOperation
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询MonitorOperation的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-25 09:48:01
 *
 ******************************************************************************/
func RegisterSelectMonitorOperation(party router.Party, path string, method func(*global.User, map[string]string) ([]model.MonitorOperation, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryMonitorOperation
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询MonitorOperation并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-25 09:48:01
 *
 ******************************************************************************/
func RegisterQueryMonitorOperation(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateMonitorOperation
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条MonitorOperation的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-25 09:48:01
 *
 ******************************************************************************/
func RegisterUpdateMonitorOperation(party router.Party, path string, method func(*global.User, *[]model.MonitorOperation) error) {

	RegisterMultiMonitorOperation(party, "PUT", path, method)
}
