// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneOPCComValueCheckRule
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条OPCComValueCheckRule的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-09-09 10:59:52
 *
 ******************************************************************************/
func RegisterOneOPCComValueCheckRule(party router.Party, httpMethod string, path string, method func(*global.User, *model.OPCComValueCheckRule) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.OPCComValueCheckRule)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfOPCComValueCheckRule
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条OPCComValueCheckRule的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-09-09 10:59:52
 *
 ******************************************************************************/
func RegisterIDOfOPCComValueCheckRule(party router.Party, httpMethod string, path string, method func(*global.User, int) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		id, err := ctx.Params().GetInt("id")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, id)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneOPCComValueCheckRule
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条OPCComValueCheckRule的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-09-09 10:59:52
 *
 ******************************************************************************/
func RegisterInsertOneOPCComValueCheckRule(party router.Party, path string, method func(*global.User, *model.OPCComValueCheckRule) error) {

	RegisterOneOPCComValueCheckRule(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneOPCComValueCheckRule
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条OPCComValueCheckRule的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-09-09 10:59:52
 *
 ******************************************************************************/
func RegisterDeleteOneOPCComValueCheckRule(party router.Party, path string, method func(*global.User, int) error) {

	RegisterIDOfOPCComValueCheckRule(party, "DELETE", path+"/{id:int}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneOPCComValueCheckRule
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条OPCComValueCheckRule的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-09-09 10:59:52
 *
 ******************************************************************************/
func RegisterSelectOneOPCComValueCheckRule(party router.Party, path string, method func(*global.User, int) (*model.OPCComValueCheckRule, error)) {

	party.Get(path+"/{id:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		id, err := ctx.Params().GetInt("id")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		result, err := method(user, id)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneOPCComValueCheckRule
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条OPCComValueCheckRule的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-09-09 10:59:52
 *
 ******************************************************************************/
func RegisterUpdateOneOPCComValueCheckRule(party router.Party, path string, method func(*global.User, *model.OPCComValueCheckRule) error) {

	RegisterOneOPCComValueCheckRule(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiOPCComValueCheckRule
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条OPCComValueCheckRule的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-09-09 10:59:52
 *
 ******************************************************************************/
func RegisterMultiOPCComValueCheckRule(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.OPCComValueCheckRule) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.OPCComValueCheckRule, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOPCComValueCheckRule
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条OPCComValueCheckRule的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-09-09 10:59:52
 *
 ******************************************************************************/
func RegisterInsertOPCComValueCheckRule(party router.Party, path string, method func(*global.User, *[]model.OPCComValueCheckRule) error) {

	RegisterMultiOPCComValueCheckRule(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOPCComValueCheckRule
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条OPCComValueCheckRule的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-09-09 10:59:52
 *
 ******************************************************************************/
func RegisterDeleteOPCComValueCheckRule(party router.Party, path string, method func(*global.User, *[]model.OPCComValueCheckRule) error) {

	RegisterMultiOPCComValueCheckRule(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOPCComValueCheckRule
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询OPCComValueCheckRule的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-09-09 10:59:52
 *
 ******************************************************************************/
func RegisterSelectOPCComValueCheckRule(party router.Party, path string, method func(*global.User, map[string]string) (interface{}, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOPCComValueCheckRule
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条OPCComValueCheckRule的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-09-09 10:59:52
 *
 ******************************************************************************/
func RegisterUpdateOPCComValueCheckRule(party router.Party, path string, method func(*global.User, *[]model.OPCComValueCheckRule) error) {

	RegisterMultiOPCComValueCheckRule(party, "PUT", path, method)
}
