// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneRPDashBoardHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条RPDashBoardHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-06 14:56:42
 *
 ******************************************************************************/
func RegisterOneRPDashBoardHead(party router.Party, httpMethod string, path string, method func(*global.User, *model.RPDashBoardHead) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil

		entity := new(model.RPDashBoardHead)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfRPDashBoardHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条RPDashBoardHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-06 14:56:42
 *
 ******************************************************************************/
func RegisterIDOfRPDashBoardHead(party router.Party, httpMethod string, path string, method func(*global.User, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		dashboardId := ctx.Params().GetString("dashboardId")

		err = method(user, dashboardId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneRPDashBoardHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条RPDashBoardHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-06 14:56:42
 *
 ******************************************************************************/
func RegisterInsertOneRPDashBoardHead(party router.Party, path string, method func(*global.User, *model.RPDashBoardHead) error) {

	RegisterOneRPDashBoardHead(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneRPDashBoardHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条RPDashBoardHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-06 14:56:42
 *
 ******************************************************************************/
func RegisterDeleteOneRPDashBoardHead(party router.Party, path string, method func(*global.User, string) error) {

	RegisterIDOfRPDashBoardHead(party, "DELETE", path+"/{dashboardId:string}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneRPDashBoardHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条RPDashBoardHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-06 14:56:42
 *
 ******************************************************************************/
func RegisterSelectOneRPDashBoardHead(party router.Party, path string, method func(*global.User, string) (*model.RPDashBoardHead, error)) {

	party.Get(path+"/{dashboardId:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		dashboardId := ctx.Params().GetString("dashboardId")
		result, err := method(user, dashboardId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneRPDashBoardHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条RPDashBoardHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-06 14:56:42
 *
 ******************************************************************************/
func RegisterUpdateOneRPDashBoardHead(party router.Party, path string, method func(*global.User, *model.RPDashBoardHead) error) {

	RegisterOneRPDashBoardHead(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiRPDashBoardHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条RPDashBoardHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-06 14:56:42
 *
 ******************************************************************************/
func RegisterMultiRPDashBoardHead(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.RPDashBoardHead) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		entities := make([]model.RPDashBoardHead, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertRPDashBoardHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条RPDashBoardHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-06 14:56:42
 *
 ******************************************************************************/
func RegisterInsertRPDashBoardHead(party router.Party, path string, method func(*global.User, *[]model.RPDashBoardHead) error) {

	RegisterMultiRPDashBoardHead(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteRPDashBoardHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条RPDashBoardHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-06 14:56:42
 *
 ******************************************************************************/
func RegisterDeleteRPDashBoardHead(party router.Party, path string, method func(*global.User, *[]model.RPDashBoardHead) error) {

	RegisterMultiRPDashBoardHead(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectRPDashBoardHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询RPDashBoardHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-06 14:56:42
 *
 ******************************************************************************/
func RegisterSelectRPDashBoardHead(party router.Party, path string, method func(*global.User, map[string]string) ([]model.RPDashBoardHead, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryRPDashBoardHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询RPDashBoardHead并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-06 14:56:42
 *
 ******************************************************************************/
func RegisterQueryRPDashBoardHead(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateRPDashBoardHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条RPDashBoardHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-06 14:56:42
 *
 ******************************************************************************/
func RegisterUpdateRPDashBoardHead(party router.Party, path string, method func(*global.User, *[]model.RPDashBoardHead) error) {

	RegisterMultiRPDashBoardHead(party, "PUT", path, method)
}
