// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneScriptItem
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条ScriptItem的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-13 13:08:46
 *
 ******************************************************************************/
func RegisterOneScriptItem(party router.Party, httpMethod string, path string, method func(*global.User, *model.ScriptItem) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.ScriptItem)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfScriptItem
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条ScriptItem的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-13 13:08:46
 *
 ******************************************************************************/
func RegisterIDOfScriptItem(party router.Party, httpMethod string, path string, method func(*global.User, string, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		script := ctx.Params().GetString("script")
		tabMapCol := ctx.Params().GetString("tabMapCol")

		err = method(user, script, tabMapCol)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneScriptItem
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条ScriptItem的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-13 13:08:46
 *
 ******************************************************************************/
func RegisterInsertOneScriptItem(party router.Party, path string, method func(*global.User, *model.ScriptItem) error) {

	RegisterOneScriptItem(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneScriptItem
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条ScriptItem的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-13 13:08:46
 *
 ******************************************************************************/
func RegisterDeleteOneScriptItem(party router.Party, path string, method func(*global.User, string, string) error) {

	RegisterIDOfScriptItem(party, "DELETE", path+"/{script:string}/{tabMapCol:string}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneScriptItem
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条ScriptItem的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-13 13:08:46
 *
 ******************************************************************************/
func RegisterSelectOneScriptItem(party router.Party, path string, method func(*global.User, string, string) (*model.ScriptItem, error)) {

	party.Get(path+"/{script:string}/{tabMapCol:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		script := ctx.Params().GetString("script")
		tabMapCol := ctx.Params().GetString("tabMapCol")
		result, err := method(user, script, tabMapCol)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneScriptItem
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条ScriptItem的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-13 13:08:46
 *
 ******************************************************************************/
func RegisterUpdateOneScriptItem(party router.Party, path string, method func(*global.User, *model.ScriptItem) error) {

	RegisterOneScriptItem(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiScriptItem
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条ScriptItem的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-13 13:08:46
 *
 ******************************************************************************/
func RegisterMultiScriptItem(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.ScriptItem) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.ScriptItem, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertScriptItem
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条ScriptItem的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-13 13:08:46
 *
 ******************************************************************************/
func RegisterInsertScriptItem(party router.Party, path string, method func(*global.User, *[]model.ScriptItem) error) {

	RegisterMultiScriptItem(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteScriptItem
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条ScriptItem的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-13 13:08:46
 *
 ******************************************************************************/
func RegisterDeleteScriptItem(party router.Party, path string, method func(*global.User, *[]model.ScriptItem) error) {

	RegisterMultiScriptItem(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectScriptItem
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询ScriptItem的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-13 13:08:46
 *
 ******************************************************************************/
func RegisterSelectScriptItem(party router.Party, path string, method func(*global.User, map[string]string) (interface{}, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateScriptItem
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条ScriptItem的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-13 13:08:46
 *
 ******************************************************************************/
func RegisterUpdateScriptItem(party router.Party, path string, method func(*global.User, *[]model.ScriptItem) error) {

	RegisterMultiScriptItem(party, "PUT", path, method)
}
