// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/utils"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/glog"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
	"strings"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterOneUser(party router.Party, httpMethod string, path string, method func(*global.User, *model.User) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.User)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterUpdatePicOneUser(party router.Party, path string, method func(*global.User, *model.User) error) {

	RegisterUserUploadMustPic(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterUserUploadMustPic
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理多条RegisterUploadMustPic的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-03-18 22:49:12
 *
 ******************************************************************************/
func RegisterUserUploadMustPic(party router.Party, httpMethod string, path string, method func(*global.User, *model.User) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		//获取通过iris.WithPostMaxMemory获取的最大上传值大小。
		maxSize := ctx.Application().ConfigurationReadOnly().GetPostMaxMemory()
		err = ctx.Request().ParseMultipartForm(maxSize)
		if err != nil {
			ctx.StatusCode(iris.StatusInternalServerError)
			ctx.WriteString(err.Error())
			return
		}
		form := ctx.Request().MultipartForm
		files := form.File["upload"]
		glog.InfoExtln("上传图片", "files：", files)

		var uploadfiles []string
		for _, file := range files {
			if !utils.ValueIsEmpty(file.Filename) {
				filename := strings.Split(file.Filename, ".")
				filetype := filename[1]
				guid := utils.MakeOrderSn(user.UserId)
				filPath := "public/uploadfile/" + guid + "." + filetype
				savePath := "web/public/uploadfile/" + guid + "." + filetype
				// 上传文件至指定目录
				err := utils.SaveUploadedFile(file, savePath)
				if err != nil {
					supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
					return
				}
				uploadfiles = append(uploadfiles, filPath)
			}
		}
		picPath := strings.Join(uploadfiles, ",")

		entity := new(model.User)
		entity.UserId = user.UserId
		entity.Avatar = picPath
		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, picPath)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterIDOfUser(party router.Party, httpMethod string, path string, method func(*global.User, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		userId := ctx.Params().GetString("userId")

		err = method(user, userId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterInsertOneUser(party router.Party, path string, method func(*global.User, *model.User) error) {

	RegisterOneUser(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterDeleteOneUser(party router.Party, path string, method func(*global.User, string) error) {

	RegisterIDOfUser(party, "DELETE", path+"/{userId:string}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterSelectOneUser(party router.Party, path string, method func(*global.User, string) (*model.User, error)) {

	party.Get(path+"/{userId:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		userId := ctx.Params().GetString("userId")
		result, err := method(user, userId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterUpdateOneUser(party router.Party, path string, method func(*global.User, *model.User) error) {

	RegisterOneUser(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterUpdateStatus(party router.Party, path string, method func(*global.User, *model.User) error) {

	RegisterOneUser(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSetPwd
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterSetPwd(party router.Party, path string, method func(*global.User, *model.SysUserPwd) (bool, error)) {

	RegisterOneUserPwd(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterOneUserPwd
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterOneUserPwd(party router.Party, httpMethod string, path string, method func(*global.User, *model.SysUserPwd) (bool, error)) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.SysUserPwd)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		ok, err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterMultiUser(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.User) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.User, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterInsertUser(party router.Party, path string, method func(*global.User, *[]model.User) error) {

	RegisterMultiUser(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterDeleteUser(party router.Party, path string, method func(*global.User, *[]model.User) error) {

	RegisterMultiUser(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterSelectUser(party router.Party, path string, method func(*global.User, map[string]string) ([]model.User, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询User并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterQueryUser(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateUser
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterUpdateUser(party router.Party, path string, method func(*global.User, *[]model.User) error) {

	RegisterMultiUser(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterGetUserInfo
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条User的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-08-20 11:11:05
 *
 ******************************************************************************/
func RegisterGetUserInfo(party router.Party, path string, method func(*global.User, string) (*model.UserViewInfo, error)) {

	party.Get(path+"/{userId:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		userId := ctx.Params().GetString("userId")
		result, err := method(user, userId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}
