// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package base

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/base"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneWorkPlaceConfig
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条WorkPlaceConfig的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-13 14:01:26
 *
 ******************************************************************************/
func RegisterOneWorkPlaceConfig(party router.Party, httpMethod string, path string, method func(*global.User, *model.WorkPlaceConfig) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil

		entity := new(model.WorkPlaceConfig)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfWorkPlaceConfig
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条WorkPlaceConfig的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-13 14:01:26
 *
 ******************************************************************************/
func RegisterIDOfWorkPlaceConfig(party router.Party, httpMethod string, path string, method func(*global.User, int) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		configNr, err := ctx.Params().GetInt("configNr")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, configNr)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneWorkPlaceConfig
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条WorkPlaceConfig的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-13 14:01:26
 *
 ******************************************************************************/
func RegisterInsertOneWorkPlaceConfig(party router.Party, path string, method func(*global.User, *model.WorkPlaceConfig) error) {

	RegisterOneWorkPlaceConfig(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneWorkPlaceConfig
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条WorkPlaceConfig的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-13 14:01:26
 *
 ******************************************************************************/
func RegisterDeleteOneWorkPlaceConfig(party router.Party, path string, method func(*global.User, int) error) {

	RegisterIDOfWorkPlaceConfig(party, "DELETE", path+"/{configNr:int}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneWorkPlaceConfig
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条WorkPlaceConfig的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-13 14:01:26
 *
 ******************************************************************************/
func RegisterSelectOneWorkPlaceConfig(party router.Party, path string, method func(*global.User, int) (*model.WorkPlaceConfig, error)) {

	party.Get(path+"/{configNr:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		configNr, err := ctx.Params().GetInt("configNr")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		result, err := method(user, configNr)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneWorkPlaceConfig
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条WorkPlaceConfig的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-13 14:01:26
 *
 ******************************************************************************/
func RegisterUpdateOneWorkPlaceConfig(party router.Party, path string, method func(*global.User, *model.WorkPlaceConfig) error) {

	RegisterOneWorkPlaceConfig(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiWorkPlaceConfig
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条WorkPlaceConfig的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-13 14:01:26
 *
 ******************************************************************************/
func RegisterMultiWorkPlaceConfig(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.WorkPlaceConfig) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		entities := make([]model.WorkPlaceConfig, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertWorkPlaceConfig
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条WorkPlaceConfig的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-13 14:01:26
 *
 ******************************************************************************/
func RegisterInsertWorkPlaceConfig(party router.Party, path string, method func(*global.User, *[]model.WorkPlaceConfig) error) {

	RegisterMultiWorkPlaceConfig(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteWorkPlaceConfig
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条WorkPlaceConfig的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-13 14:01:26
 *
 ******************************************************************************/
func RegisterDeleteWorkPlaceConfig(party router.Party, path string, method func(*global.User, *[]model.WorkPlaceConfig) error) {

	RegisterMultiWorkPlaceConfig(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectWorkPlaceConfig
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询WorkPlaceConfig的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-13 14:01:26
 *
 ******************************************************************************/
func RegisterSelectWorkPlaceConfig(party router.Party, path string, method func(*global.User, map[string]string) ([]model.WorkPlaceConfig, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryWorkPlaceConfig
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询WorkPlaceConfig并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-13 14:01:26
 *
 ******************************************************************************/
func RegisterQueryWorkPlaceConfig(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateWorkPlaceConfig
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条WorkPlaceConfig的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-01-13 14:01:26
 *
 ******************************************************************************/
func RegisterUpdateWorkPlaceConfig(party router.Party, path string, method func(*global.User, *[]model.WorkPlaceConfig) error) {

	RegisterMultiWorkPlaceConfig(party, "PUT", path, method)
}
