// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package jit

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/jit"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
	"strconv"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOnePackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条PackOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterOnePackOrder(party router.Party, httpMethod string, path string, method func(*global.User, *model.PackOrder) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.PackOrder)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfPackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条PackOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterIDOfPackOrder(party router.Party, httpMethod string, path string, method func(*global.User, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		packOrderId := ctx.Params().GetString("packOrderId")

		err = method(user, packOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOnePackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条PackOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterInsertOnePackOrder(party router.Party, path string, method func(*global.User, *model.PackOrder) error) {

	RegisterOnePackOrder(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOnePackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条PackOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterDeleteOnePackOrder(party router.Party, path string, method func(*global.User, string) error) {

	RegisterIDOfPackOrder(party, "DELETE", path+"/{packOrderId:string}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOnePackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条PackOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterSelectOnePackOrder(party router.Party, path string, method func(*global.User, string) (*model.PackOrder, error)) {

	party.Get(path+"/{packOrderId:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		packOrderId := ctx.Params().GetString("packOrderId")
		result, err := method(user, packOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOnePackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条PackOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterUpdateOnePackOrder(party router.Party, path string, method func(*global.User, *model.PackOrder) error) {

	RegisterOnePackOrder(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiPackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条PackOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterMultiPackOrder(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.PackOrder) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.PackOrder, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertPackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条PackOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterInsertPackOrder(party router.Party, path string, method func(*global.User, *[]model.PackOrder) error) {

	RegisterMultiPackOrder(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeletePackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条PackOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterDeletePackOrder(party router.Party, path string, method func(*global.User, *[]model.PackOrder) error) {

	RegisterMultiPackOrder(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectPackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询PackOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterSelectPackOrder(party router.Party, path string, method func(*global.User, map[string]string) ([]model.PackOrder, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryPackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询PackOrder并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterQueryPackOrder(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdatePackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条PackOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterUpdatePackOrder(party router.Party, path string, method func(*global.User, *[]model.PackOrder) error) {

	RegisterMultiPackOrder(party, "PUT", path, method)
}


/******************************************************************************
 *
 * @Function Name            : RegisterScanBarcode
 *-----------------------------------------------------------------------------
 *
 * @Description              : 扫描工单条码进包
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : zhangxin
 *
 * @Date                     : 2021-10-22
 *
 ******************************************************************************/
func RegisterScanBarcode(party router.Party, path string, method func(user *global.User, barcode string, workplaceNr int) (*model.PackOrderInfo, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		barcode := ctx.URLParam("barcode")
		if barcode == "" {
			supports.Error(ctx, iris.StatusBadRequest, "未获取到工单条码", nil)
			return
		}
		workplaceNrStr := ctx.URLParam("workPlaceNr")
		var (
			workplaceNr int
			err error
		)
		if workplaceNrStr == "" {
			workplaceNr = 0
		} else {
			workplaceNr, err = strconv.Atoi(workplaceNrStr)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, "工位数据格式错误", nil)
				return
			}
		}
		result, err := method(user, barcode, workplaceNr)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterClosePackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 关闭包装单
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : zhangxin
 *
 * @Date                     : 2021-10-22
 *
 ******************************************************************************/
func RegisterClosePackOrder(party router.Party, path string, method func(user *global.User, packOrderId string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		type ClosePackOrderData struct {
			PackOrderId string `json:"packOrderId"`
		}
		var data ClosePackOrderData
		if err := ctx.ReadJSON(&data); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "解析参数错误" + err.Error(), nil)
			return
		}
		if data.PackOrderId == "" {
			supports.Error(ctx, iris.StatusBadRequest, "未获取到包装单ID", nil)
			return
		}
		err := method(user, data.PackOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterSplitPackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 拆包
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : zhangxin
 *
 * @Date                     : 2021-11-09
 *
 ******************************************************************************/
func RegisterSplitPackOrder(party router.Party, path string, method func(user *global.User, packOrderId string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		type Req struct {
			PackOrderId string `json:"packOrderId"`
		}
		var data Req
		if err := ctx.ReadJSON(&data); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "解析参数错误" + err.Error(), nil)
			return
		}
		if data.PackOrderId == "" {
			supports.Error(ctx, iris.StatusBadRequest, "未获取到包装单ID", nil)
			return
		}
		err := method(user, data.PackOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOperatePackOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条PackOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterSelectOperatePackOrder(party router.Party, path string, method func(*global.User, string) (string, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		location := ctx.URLParam("location")
		if location == "" {
			supports.Error(ctx, iris.StatusBadRequest, "未获取到包装地点数据", nil)
			return
		}
		result, err := method(user, location)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterSplitPackOrderBySerialOrderId
 *-----------------------------------------------------------------------------
 *
 * @Description              : 拆包
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : zhangxin
 *
 * @Date                     : 2021-11-09
 *
 ******************************************************************************/
func RegisterSplitPackOrderBySerialOrderId(party router.Party, path string, method func(user *global.User, packOrderId string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		type Req struct {
			SerialOrderId string `json:"serialOrderId"`
		}
		var data Req
		if err := ctx.ReadJSON(&data); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "解析参数错误" + err.Error(), nil)
			return
		}
		if data.SerialOrderId == "" {
			supports.Error(ctx, iris.StatusBadRequest, "未获取到工单ID", nil)
			return
		}
		err := method(user, data.SerialOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterScanBarcodeOutLine
 *-----------------------------------------------------------------------------
 *
 * @Description              : 线外打包扫描工单条码进包
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : zhangxin
 *
 * @Date                     : 2021-10-22
 *
 ******************************************************************************/
func RegisterScanBarcodeOutLine(party router.Party, path string, method func(user *global.User, barcode string) (*model.PackOrderInfo, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		barcode := ctx.URLParam("barcode")
		if barcode == "" {
			supports.Error(ctx, iris.StatusBadRequest, "未获取到工单条码", nil)
			return
		}
		result, err := method(user, barcode)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterRepairPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 修复打包写入plc的错误
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : zhangxin
 *
 * @Date                     : 2022-02-10
 *
 ******************************************************************************/
func RegisterRepairPlcError(party router.Party, path string, method func(user *global.User, packOrderId string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		type Req struct {
			PackOrderId string
		}
		var data Req
		if err := ctx.ReadJSON(&data); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if data.PackOrderId == "" {
			supports.Error(ctx, iris.StatusBadRequest, "未获取到包装单ID", nil)
			return
		}
		err := method(user, data.PackOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}