// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package jit

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/jit"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOnePackOrderItemLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条PackOrderItemLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterOnePackOrderItemLst(party router.Party, httpMethod string, path string, method func(*global.User, *model.PackOrderItemLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.PackOrderItemLst)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfPackOrderItemLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条PackOrderItemLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterIDOfPackOrderItemLst(party router.Party, httpMethod string, path string, method func(*global.User, string, int) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		packOrderId := ctx.Params().GetString("packOrderId")
		pos, err := ctx.Params().GetInt("pos")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, packOrderId, pos)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOnePackOrderItemLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条PackOrderItemLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterInsertOnePackOrderItemLst(party router.Party, path string, method func(*global.User, *model.PackOrderItemLst) error) {

	RegisterOnePackOrderItemLst(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOnePackOrderItemLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条PackOrderItemLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterDeleteOnePackOrderItemLst(party router.Party, path string, method func(*global.User, string, int) error) {

	RegisterIDOfPackOrderItemLst(party, "DELETE", path+"/{packOrderId:string}/{pos:int}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOnePackOrderItemLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条PackOrderItemLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterSelectOnePackOrderItemLst(party router.Party, path string, method func(*global.User, string, int) (*model.PackOrderItemLst, error)) {

	party.Get(path+"/{packOrderId:string}/{pos:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		packOrderId := ctx.Params().GetString("packOrderId")
		pos, err := ctx.Params().GetInt("pos")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		result, err := method(user, packOrderId, pos)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOnePackOrderItemLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条PackOrderItemLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterUpdateOnePackOrderItemLst(party router.Party, path string, method func(*global.User, *model.PackOrderItemLst) error) {

	RegisterOnePackOrderItemLst(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiPackOrderItemLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条PackOrderItemLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterMultiPackOrderItemLst(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.PackOrderItemLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.PackOrderItemLst, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertPackOrderItemLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条PackOrderItemLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterInsertPackOrderItemLst(party router.Party, path string, method func(*global.User, *[]model.PackOrderItemLst) error) {

	RegisterMultiPackOrderItemLst(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeletePackOrderItemLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条PackOrderItemLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterDeletePackOrderItemLst(party router.Party, path string, method func(*global.User, *[]model.PackOrderItemLst) error) {

	RegisterMultiPackOrderItemLst(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectPackOrderItemLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询PackOrderItemLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterSelectPackOrderItemLst(party router.Party, path string, method func(*global.User, map[string]string) ([]model.PackOrderItemLst, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryPackOrderItemLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询PackOrderItemLst并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterQueryPackOrderItemLst(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdatePackOrderItemLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条PackOrderItemLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 10:26:40
 *
 ******************************************************************************/
func RegisterUpdatePackOrderItemLst(party router.Party, path string, method func(*global.User, *[]model.PackOrderItemLst) error) {

	RegisterMultiPackOrderItemLst(party, "PUT", path, method)
}
