// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package jit

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/jit"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOnePackOrderPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条PackOrderPlcError的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-10 14:47:04
 *
 ******************************************************************************/
func RegisterOnePackOrderPlcError(party router.Party, httpMethod string, path string, method func(*global.User, *model.PackOrderPlcError) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil

		entity := new(model.PackOrderPlcError)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfPackOrderPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条PackOrderPlcError的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-10 14:47:04
 *
 ******************************************************************************/
func RegisterIDOfPackOrderPlcError(party router.Party, httpMethod string, path string, method func(*global.User, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		packOrderId := ctx.Params().GetString("packOrderId")

		err = method(user, packOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOnePackOrderPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条PackOrderPlcError的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-10 14:47:04
 *
 ******************************************************************************/
func RegisterInsertOnePackOrderPlcError(party router.Party, path string, method func(*global.User, *model.PackOrderPlcError) error) {

	RegisterOnePackOrderPlcError(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOnePackOrderPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条PackOrderPlcError的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-10 14:47:04
 *
 ******************************************************************************/
func RegisterDeleteOnePackOrderPlcError(party router.Party, path string, method func(*global.User, string) error) {

	RegisterIDOfPackOrderPlcError(party, "DELETE", path+"/{packOrderId:string}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOnePackOrderPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条PackOrderPlcError的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-10 14:47:04
 *
 ******************************************************************************/
func RegisterSelectOnePackOrderPlcError(party router.Party, path string, method func(*global.User, string) (*model.PackOrderPlcError, error)) {

	party.Get(path+"/{packOrderId:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		packOrderId := ctx.Params().GetString("packOrderId")
		result, err := method(user, packOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOnePackOrderPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条PackOrderPlcError的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-10 14:47:04
 *
 ******************************************************************************/
func RegisterUpdateOnePackOrderPlcError(party router.Party, path string, method func(*global.User, *model.PackOrderPlcError) error) {

	RegisterOnePackOrderPlcError(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiPackOrderPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条PackOrderPlcError的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-10 14:47:04
 *
 ******************************************************************************/
func RegisterMultiPackOrderPlcError(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.PackOrderPlcError) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		entities := make([]model.PackOrderPlcError, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertPackOrderPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条PackOrderPlcError的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-10 14:47:04
 *
 ******************************************************************************/
func RegisterInsertPackOrderPlcError(party router.Party, path string, method func(*global.User, *[]model.PackOrderPlcError) error) {

	RegisterMultiPackOrderPlcError(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeletePackOrderPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条PackOrderPlcError的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-10 14:47:04
 *
 ******************************************************************************/
func RegisterDeletePackOrderPlcError(party router.Party, path string, method func(*global.User, *[]model.PackOrderPlcError) error) {

	RegisterMultiPackOrderPlcError(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectPackOrderPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询PackOrderPlcError的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-10 14:47:04
 *
 ******************************************************************************/
func RegisterSelectPackOrderPlcError(party router.Party, path string, method func(*global.User, map[string]string) ([]model.PackOrderPlcError, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryPackOrderPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询PackOrderPlcError并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-10 14:47:04
 *
 ******************************************************************************/
func RegisterQueryPackOrderPlcError(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdatePackOrderPlcError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条PackOrderPlcError的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-10 14:47:04
 *
 ******************************************************************************/
func RegisterUpdatePackOrderPlcError(party router.Party, path string, method func(*global.User, *[]model.PackOrderPlcError) error) {

	RegisterMultiPackOrderPlcError(party, "PUT", path, method)
}
