// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package me

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/me"
	"LAPP_ACURA_MOM_BACKEND/utils"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
	"strings"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterOneProductFamily(party router.Party, httpMethod string, path string, method func(*global.User, *model.ProductFamily) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil

		entity := new(model.ProductFamily)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterAssociationProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 配置关联ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterAssociationProductFamily(party router.Party, path string, method func(*global.User, *model.ProductFamily) error) {
	party.Handle("PUT", path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil

		entity := new(model.ProductFamily)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterIDOfProductFamily(party router.Party, httpMethod string, path string, method func(*global.User, string, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		productFamilyId := ctx.Params().GetString("productFamilyId")
		projectId := ctx.Params().GetString("projectId")
		err = method(user, productFamilyId, projectId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterInsertOneProductFamily(party router.Party, path string, method func(*global.User, *model.ProductFamily) error) {

	RegisterOneProductFamily(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterDeleteOneProductFamily(party router.Party, path string, method func(*global.User, string, string) error) {

	RegisterIDOfProductFamily(party, "DELETE", path+"/{projectId:string}/{productFamilyId:string}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterSelectOneProductFamily(party router.Party, path string, method func(*global.User, string) (*model.ProductFamily, error)) {

	party.Get(path+"/{projectId:string}/{productFamilyId:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		productFamilyId := ctx.Params().GetString("productFamilyId")
		result, err := method(user, productFamilyId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterUpdateOneProductFamily(party router.Party, path string, method func(*global.User, *model.ProductFamily) error) {

	RegisterOneProductFamily(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterMultiProductFamily(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.ProductFamily) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		entities := make([]model.ProductFamily, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterInsertProductFamily(party router.Party, path string, method func(*global.User, *[]model.ProductFamily) error) {

	RegisterMultiProductFamily(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterDeleteProductFamily(party router.Party, path string, method func(*global.User, *[]model.ProductFamily) error) {

	RegisterMultiProductFamily(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterSelectProductFamily(party router.Party, path string, method func(*global.User, map[string]string) ([]model.ProductFamily, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询ProductFamily并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterQueryProductFamily(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterUpdateProductFamily(party router.Party, path string, method func(*global.User, *[]model.ProductFamily) error) {

	RegisterMultiProductFamily(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneProductFamily
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条ProductFamily的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-19 14:04:53
 *
 ******************************************************************************/
func RegisterGeneratePackTemplateByProductFamily(party router.Party, path string, method func(*global.User, string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		type GenerateData struct {
			ProductFamilyId string `json:"productFamilyId"`
		}
		var entity GenerateData
		if err := ctx.ReadJSON(&entity); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "解析参数错误:"+err.Error(), nil)
			return
		}

		err := method(user, entity.ProductFamilyId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterProductFamilyUploadExcel
 *-----------------------------------------------------------------------------
 *
 * @Description              : excel导入数据
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : zhangxin
 *
 * @Date                     : 2021-12-22
 *
 ******************************************************************************/
func RegisterProductFamilyUploadExcel(party router.Party, path string, method func(*global.User, string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		var err error = nil
		_, info, err := ctx.FormFile("uploadfile")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "获取上传文件出错,err: "+err.Error(), nil)
			return
		}

		guid := utils.MakeOrderSn("ME")
		filenameLi := strings.Split(info.Filename, ".")
		filename := filenameLi[0]
		savePath := "web/public/uploadxlsx/" + filename + guid + ".xlsx"
		err = utils.SaveUploadedFile(info, savePath)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "保存上传文件出错,err: "+err.Error(), nil)
			return
		}

		err = method(user, savePath)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}
