// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package me

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	baseModel "LAPP_ACURA_MOM_BACKEND/models/base"
	model "LAPP_ACURA_MOM_BACKEND/models/me"
	"LAPP_ACURA_MOM_BACKEND/utils"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
	"strings"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneProductFamilyRelate
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条ProductFamilyRelate的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 14:45:56
 *
 ******************************************************************************/
func RegisterOneProductFamilyRelate(party router.Party, httpMethod string, path string, method func(*global.User, *model.ProductFamilyRelate) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil

		entity := new(model.ProductFamilyRelate)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfProductFamilyRelate
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条ProductFamilyRelate的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 14:45:56
 *
 ******************************************************************************/
func RegisterIDOfProductFamilyRelate(party router.Party, httpMethod string, path string, method func(*global.User, string, string, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		productFamilyId := ctx.Params().GetString("productFamilyId")
		projectId := ctx.Params().GetString("projectId")
		productId := ctx.Params().GetString("productId")

		err = method(user, productFamilyId, projectId, productId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneProductFamilyRelate
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条ProductFamilyRelate的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 14:45:56
 *
 ******************************************************************************/
func RegisterInsertOneProductFamilyRelate(party router.Party, path string, method func(*global.User, *model.ProductFamilyRelate) error) {

	RegisterOneProductFamilyRelate(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneProductFamilyRelate
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条ProductFamilyRelate的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 14:45:56
 *
 ******************************************************************************/
func RegisterDeleteOneProductFamilyRelate(party router.Party, path string, method func(*global.User, string, string, string) error) {

	RegisterIDOfProductFamilyRelate(party, "DELETE", path+"/{productFamilyId:string}/{projectId:string}/{productId:string}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneProductFamilyRelate
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条ProductFamilyRelate的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 14:45:56
 *
 ******************************************************************************/
func RegisterSelectOneProductFamilyRelate(party router.Party, path string, method func(*global.User, string, string, string) (*model.ProductFamilyRelate, error)) {

	party.Get(path+"/{productFamilyId:string}/{projectId:string}/{productId:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		productFamilyId := ctx.Params().GetString("productFamilyId")
		projectId := ctx.Params().GetString("projectId")
		productId := ctx.Params().GetString("productId")
		result, err := method(user, productFamilyId, projectId, productId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneProductFamilyRelate
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条ProductFamilyRelate的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 14:45:56
 *
 ******************************************************************************/
func RegisterUpdateOneProductFamilyRelate(party router.Party, path string, method func(*global.User, *model.ProductFamilyRelate) error) {

	RegisterOneProductFamilyRelate(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiProductFamilyRelate
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条ProductFamilyRelate的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 14:45:56
 *
 ******************************************************************************/
func RegisterMultiProductFamilyRelate(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.ProductFamilyRelate) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		entities := make([]model.ProductFamilyRelate, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertProductFamilyRelate
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条ProductFamilyRelate的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 14:45:56
 *
 ******************************************************************************/
func RegisterInsertProductFamilyRelate(party router.Party, path string, method func(*global.User, *[]model.ProductFamilyRelate) error) {

	RegisterMultiProductFamilyRelate(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteProductFamilyRelate
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条ProductFamilyRelate的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 14:45:56
 *
 ******************************************************************************/
func RegisterDeleteProductFamilyRelate(party router.Party, path string, method func(*global.User, *[]model.ProductFamilyRelate) error) {

	RegisterMultiProductFamilyRelate(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectProductFamilyRelate
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询ProductFamilyRelate的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 14:45:56
 *
 ******************************************************************************/
func RegisterSelectProductFamilyRelate(party router.Party, path string, method func(*global.User, map[string]string) ([]model.ProductFamilyRelate, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryProductFamilyRelate
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询ProductFamilyRelate并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 14:45:56
 *
 ******************************************************************************/
func RegisterQueryProductFamilyRelate(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateProductFamilyRelate
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条ProductFamilyRelate的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 14:45:56
 *
 ******************************************************************************/
func RegisterUpdateProductFamilyRelate(party router.Party, path string, method func(*global.User, *[]model.ProductFamilyRelate) error) {

	RegisterMultiProductFamilyRelate(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterProductFamilyUploadExcel
 *-----------------------------------------------------------------------------
 *
 * @Description              : excel导入数据
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : zhangxin
 *
 * @Date                     : 2021-12-22
 *
 ******************************************************************************/
func RegisterProductFamilyRelateUploadExcel(party router.Party, path string, method func(*global.User, string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		var err error = nil
		_, info, err := ctx.FormFile("uploadfile")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "获取上传文件出错,err: "+err.Error(), nil)
			return
		}

		guid := utils.MakeOrderSn("ME")
		filenameLi := strings.Split(info.Filename, ".")
		filename := filenameLi[0]
		savePath := "web/public/uploadxlsx/" + filename + guid + ".xlsx"
		err = utils.SaveUploadedFile(info, savePath)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "保存上传文件出错,err: "+err.Error(), nil)
			return
		}

		err = method(user, savePath)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterProductFamilySelectWorkLine
 *-----------------------------------------------------------------------------
 *
 * @Description              : 通过派生查询产线
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : zhangxin
 *
 * @Date                     : 2021-12-22
 *
 ******************************************************************************/
func RegisterProductFamilySelectWorkLine(party router.Party, path string, method func(*global.User, string) ([]baseModel.WorkLine, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		productFamilyId := ctx.URLParam("productFamilyId")
		if productFamilyId == "" {
			supports.Error(ctx, iris.StatusBadRequest, "未获取到派生ID", nil)
			return
		}
		result, err := method(user, productFamilyId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}
