// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package om

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	model "LAPP_ACURA_MOM_BACKEND/models/om"
	"LAPP_ACURA_MOM_BACKEND/utils"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneDemandHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterOneDemandHead(party router.Party, httpMethod string, path string, method func(*global.User, *model.DemandHead) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.DemandHead)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfDemandHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterIDOfDemandHead(party router.Party, httpMethod string, path string, method func(*global.User, int) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		fileNr, err := ctx.Params().GetInt("fileNr")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, fileNr)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneDemandHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterInsertOneDemandHead(party router.Party, path string, method func(*global.User, *model.DemandHead) error) {

	RegisterOneDemandHead(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneDemandHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterDeleteOneDemandHead(party router.Party, path string, method func(*global.User, int) error) {

	RegisterIDOfDemandHead(party, "DELETE", path+"/{fileNr:int}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneDemandHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterSelectOneDemandHead(party router.Party, path string, method func(*global.User, int) (*model.DemandHead, error)) {

	party.Get(path+"/{fileNr:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		fileNr, err := ctx.Params().GetInt("fileNr")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		result, err := method(user, fileNr)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneDemandHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterUpdateOneDemandHead(party router.Party, path string, method func(*global.User, *model.DemandHead) error) {

	RegisterOneDemandHead(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiDemandHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterMultiDemandHead(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.DemandHead) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.DemandHead, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertDemandHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterInsertDemandHead(party router.Party, path string, method func(*global.User, *[]model.DemandHead) error) {

	RegisterMultiDemandHead(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteDemandHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterDeleteDemandHead(party router.Party, path string, method func(*global.User, *[]model.DemandHead) error) {

	RegisterMultiDemandHead(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectDemandHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterSelectDemandHead(party router.Party, path string, method func(*global.User, map[string]string) (interface{}, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateDemandHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterUpdateDemandHead(party router.Party, path string, method func(*global.User, *[]model.DemandHead) error) {

	RegisterMultiDemandHead(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterUploadDemandHeadExcel
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterUploadDemandHeadExcel(party router.Party, path string, method func(*global.User, string, string, string) error) {

	RegisterUploadExcel(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterOneDemandHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条DemandHead的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-04-21 10:27:00
 *
 ******************************************************************************/
func RegisterUploadExcel(party router.Party, httpMethod string, path string, method func(*global.User, string, string, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		source := ctx.URLParam("Source")
		if source == "" || (source != model.ParseSourceMES && source != model.ParseSourceInterval) {
			supports.Error(ctx, iris.StatusBadRequest, "缺少source关键字", nil)
			return
		}

		var err error = nil
		//form := ctx.Request().MultipartForm
		_, info, err := ctx.FormFile("uploadfile")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "获取上传文件出错,err: "+err.Error(), nil)
			return
		}

		guid := utils.MakeOrderSn("OM")
		savePath := "web/public/uploadxlsx/" + info.Filename + guid + ".xlsx"
		err = utils.SaveUploadedFile(info, savePath)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "保存上传文件出错,err: "+err.Error(), nil)
			return
		}

		err = method(user, savePath, info.Filename, source)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}
