// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package om

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/om"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneSerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条SerialOrderDisassemble的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterOneSerialOrderDisassemble(party router.Party, httpMethod string, path string, method func(*global.User, *model.SerialOrderDisassemble) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil

		entity := new(model.SerialOrderDisassemble)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfSerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条SerialOrderDisassemble的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterIDOfSerialOrderDisassemble(party router.Party, httpMethod string, path string, method func(*global.User, string, int) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		serialOrderId := ctx.Params().GetString("serialOrderId")
		pos, err := ctx.Params().GetInt("pos")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, serialOrderId, pos)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneSerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条SerialOrderDisassemble的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterInsertOneSerialOrderDisassemble(party router.Party, path string, method func(*global.User, *model.SerialOrderDisassemble) error) {

	RegisterOneSerialOrderDisassemble(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneSerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条SerialOrderDisassemble的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterDeleteOneSerialOrderDisassemble(party router.Party, path string, method func(*global.User, string, int) error) {

	RegisterIDOfSerialOrderDisassemble(party, "DELETE", path+"/{serialOrderId:string}/{pos:int}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneSerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条SerialOrderDisassemble的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterSelectOneSerialOrderDisassemble(party router.Party, path string, method func(*global.User, string, int) (*model.SerialOrderDisassemble, error)) {

	party.Get(path+"/{serialOrderId:string}/{pos:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		serialOrderId := ctx.Params().GetString("serialOrderId")
		pos, err := ctx.Params().GetInt("pos")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		result, err := method(user, serialOrderId, pos)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneSerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条SerialOrderDisassemble的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterUpdateOneSerialOrderDisassemble(party router.Party, path string, method func(*global.User, *model.SerialOrderDisassemble) error) {

	RegisterOneSerialOrderDisassemble(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiSerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条SerialOrderDisassemble的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterMultiSerialOrderDisassemble(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.SerialOrderDisassemble) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		entities := make([]model.SerialOrderDisassemble, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertSerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条SerialOrderDisassemble的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterInsertSerialOrderDisassemble(party router.Party, path string, method func(*global.User, *[]model.SerialOrderDisassemble) error) {

	RegisterMultiSerialOrderDisassemble(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteSerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条SerialOrderDisassemble的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterDeleteSerialOrderDisassemble(party router.Party, path string, method func(*global.User, *[]model.SerialOrderDisassemble) error) {

	RegisterMultiSerialOrderDisassemble(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectSerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询SerialOrderDisassemble的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterSelectSerialOrderDisassemble(party router.Party, path string, method func(*global.User, map[string]string) ([]model.SerialOrderDisassemble, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQuerySerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询SerialOrderDisassemble并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterQuerySerialOrderDisassemble(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateSerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条SerialOrderDisassemble的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterUpdateSerialOrderDisassemble(party router.Party, path string, method func(*global.User, *[]model.SerialOrderDisassemble) error) {

	RegisterMultiSerialOrderDisassemble(party, "PUT", path, method)
}


/******************************************************************************
 *
 * @Function Name            : RegisterOneSerialOrderDisassemble
 *-----------------------------------------------------------------------------
 *
 * @Description              : 记录返修拆解
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : zhangxin
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterInsertSerialOrderRecord(party router.Party, path string, method func(*global.User, string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		type Req struct {
			SerialOrderId string `json:"serialOrderId"`
		}
		entity := new(Req)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			supports.Error(ctx, iris.StatusBadRequest, "未获取到工单ID", nil)
			return
		}

		err = method(user, entity.SerialOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}


/******************************************************************************
 *
 * @Function Name            : RegisterSelectSerialOrderRecord
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条SerialOrderDisassemble的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-28 09:55:48
 *
 ******************************************************************************/
func RegisterSelectSerialOrderRecord(party router.Party, path string, method func(*global.User, string) ([]model.SerialOrderDisassemble, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		serialOrderId := ctx.URLParam("serialOrderId")
		if serialOrderId == "" {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, serialOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}