// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package om

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	model "LAPP_ACURA_MOM_BACKEND/models/om"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneSerialOrderPOLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条SerialOrderPOLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-29 17:19:37
 *
 ******************************************************************************/
func RegisterOneSerialOrderPOLst(party router.Party, httpMethod string, path string, method func(*global.User, *model.SerialOrderPOLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.SerialOrderPOLst)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfSerialOrderPOLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条SerialOrderPOLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-29 17:19:37
 *
 ******************************************************************************/
func RegisterIDOfSerialOrderPOLst(party router.Party, httpMethod string, path string, method func(*global.User, string, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		serialOrderId := ctx.Params().GetString("serialOrderId")
		pO:= ctx.Params().GetString("pO")
		err = method(user, serialOrderId, pO)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneSerialOrderPOLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条SerialOrderPOLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-29 17:19:37
 *
 ******************************************************************************/
func RegisterInsertOneSerialOrderPOLst(party router.Party, path string, method func(*global.User, *model.SerialOrderPOLst) error) {

	RegisterOneSerialOrderPOLst(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneSerialOrderPOLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条SerialOrderPOLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-29 17:19:37
 *
 ******************************************************************************/
func RegisterDeleteOneSerialOrderPOLst(party router.Party, path string, method func(*global.User, string, string) error) {

	RegisterIDOfSerialOrderPOLst(party, "DELETE", path+"/{serialOrderId:string}/{pO:int}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneSerialOrderPOLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条SerialOrderPOLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-29 17:19:37
 *
 ******************************************************************************/
func RegisterSelectOneSerialOrderPOLst(party router.Party, path string, method func(*global.User, string, string) (*model.SerialOrderPOLst, error)) {

	party.Get(path+"/{serialOrderId:string}/{pO:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		serialOrderId := ctx.Params().GetString("serialOrderId")
		pO := ctx.Params().GetString("pO")
		result, err := method(user, serialOrderId, pO)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneSerialOrderPOLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条SerialOrderPOLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-29 17:19:37
 *
 ******************************************************************************/
func RegisterUpdateOneSerialOrderPOLst(party router.Party, path string, method func(*global.User, *model.SerialOrderPOLst) error) {

	RegisterOneSerialOrderPOLst(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiSerialOrderPOLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条SerialOrderPOLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-29 17:19:37
 *
 ******************************************************************************/
func RegisterMultiSerialOrderPOLst(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.SerialOrderPOLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.SerialOrderPOLst, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertSerialOrderPOLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条SerialOrderPOLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-29 17:19:37
 *
 ******************************************************************************/
func RegisterInsertSerialOrderPOLst(party router.Party, path string, method func(*global.User, *[]model.SerialOrderPOLst) error) {

	RegisterMultiSerialOrderPOLst(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteSerialOrderPOLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条SerialOrderPOLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-29 17:19:37
 *
 ******************************************************************************/
func RegisterDeleteSerialOrderPOLst(party router.Party, path string, method func(*global.User, *[]model.SerialOrderPOLst) error) {

	RegisterMultiSerialOrderPOLst(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectSerialOrderPOLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询SerialOrderPOLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-29 17:19:37
 *
 ******************************************************************************/
func RegisterSelectSerialOrderPOLst(party router.Party, path string, method func(*global.User, map[string]string) (interface{}, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateSerialOrderPOLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条SerialOrderPOLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-29 17:19:37
 *
 ******************************************************************************/
func RegisterUpdateSerialOrderPOLst(party router.Party, path string, method func(*global.User, *[]model.SerialOrderPOLst) error) {

	RegisterMultiSerialOrderPOLst(party, "PUT", path, method)
}
