// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package pln

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/pln"
	"LAPP_ACURA_MOM_BACKEND/utils"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
	"strings"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterOneCustOrder(party router.Party, httpMethod string, path string, method func(*global.User, *model.CustOrder) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.CustOrder)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterIDOfCustOrder(party router.Party, httpMethod string, path string, method func(*global.User, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		custOrderId := ctx.Params().GetString("custOrderId")

		err = method(user, custOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterInsertOneCustOrder(party router.Party, path string, method func(*global.User, *model.CustOrder) error) {

	RegisterOneCustOrder(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterDeleteOneCustOrder(party router.Party, path string, method func(*global.User, string) error) {

	RegisterIDOfCustOrder(party, "DELETE", path+"/{custOrderId:string}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterCancelCustOrder(party router.Party, path string, method func(*global.User, string) error) {

	party.Handle("DELETE", path+"/{custOrderId:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		custOrderId := ctx.Params().GetString("custOrderId")

		err = method(user, custOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterSelectOneCustOrder(party router.Party, path string, method func(*global.User, string) (*model.CustOrder, error)) {

	party.Get(path+"/{custOrderId:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		custOrderId := ctx.Params().GetString("custOrderId")
		result, err := method(user, custOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterUpdateOneCustOrder(party router.Party, path string, method func(*global.User, *model.CustOrder) error) {

	RegisterOneCustOrder(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterMultiCustOrder(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.CustOrder) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.CustOrder, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterInsertCustOrder(party router.Party, path string, method func(*global.User, *[]model.CustOrder) error) {

	RegisterMultiCustOrder(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterDeleteCustOrder(party router.Party, path string, method func(*global.User, *[]model.CustOrder) error) {

	RegisterMultiCustOrder(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterSelectCustOrder(party router.Party, path string, method func(*global.User, map[string]string) ([]model.CustOrder, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询CustOrder并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterQueryCustOrder(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterUpdateCustOrder(party router.Party, path string, method func(*global.User, *[]model.CustOrder) error) {

	RegisterMultiCustOrder(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterPlanCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个计划CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : zhangxin
 *
 * @Date                     : 2021-10-29
 *
 ******************************************************************************/
func RegisterPlanCustOrder(party router.Party, path string, method func(user *global.User, entity *model.CustOrder) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var entity model.CustOrder
		if err := ctx.ReadJSON(&entity); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		err := method(user, &entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterLockCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个锁定CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : zhangxin
 *
 * @Date                     : 2021-10-29
 *
 ******************************************************************************/
func RegisterLockCustOrder(party router.Party, path string, method func(user *global.User, custOrderIdLi []string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var entity []string
		if err := ctx.ReadJSON(&entity); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		err := method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterCancelCustOrderStatus
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个取消单条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterCancelCustOrderStatus(party router.Party, path string, method func(*global.User, string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		type Req struct {
			CustOrderId string `json:"custOrderId"`
		}
		var data Req
		if err := ctx.ReadJSON(&data); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		err := method(user, data.CustOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterCancelCustOrderQty
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个取消单条CustOrder数量的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterCancelCustOrderQty(party router.Party, path string, method func(*global.User, string, int) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		type Req struct {
			CustOrderId string `json:"custOrderId"`
			Qty         int    `json:"qty"`
		}
		var data Req
		if err := ctx.ReadJSON(&data); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		err := method(user, data.CustOrderId, data.Qty)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterWithdrawCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个回撤单条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterWithdrawCustOrder(party router.Party, path string, method func(*global.User, string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		type Req struct {
			CustOrderId string `json:"custOrderId"`
		}
		var data Req
		if err := ctx.ReadJSON(&data); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		err := method(user, data.CustOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterDashboard
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个回撤单条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterDashboard(party router.Party, path string, method func(*global.User, string) (interface{}, error)) {

	party.Get(path, func(ctx iris.Context) {
		plantNr, err := ctx.URLParamInt("plantNr")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		workLineId := ctx.URLParam("workLineId")
		if plantNr == 0 || workLineId == "" {
			supports.Error(ctx, iris.StatusBadRequest, "参数不全", nil)
			return
		}
		user := &global.User{
			PlantNr: plantNr,
			UserId: "Dashboard",
		}
		data, err := method(user, workLineId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, data)
	})
}


/******************************************************************************
 *
 * @Function Name            : RegisterSelectCanCancelQty
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个回撤单条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterSelectCanCancelQty(party router.Party, path string, method func(user *global.User, custOrderId string) (int, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		custOrderId := ctx.URLParam("custOrderId")
		if custOrderId == "" {
			supports.Error(ctx, iris.StatusBadRequest, "未获取到客户订单ID", nil)
			return
		}
		data, err := method(user, custOrderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, data)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterSplitOrderQty
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个回撤单条CustOrder的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-22 13:50:40
 *
 ******************************************************************************/
func RegisterSplitOrderQty(party router.Party, path string, method func(user *global.User, custOrderId string, qty int) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		type Req struct {
			CustOrderId string `json:"custOrderId"`
			Qty int `json:"qty"`
		}
		var data Req
		if err := ctx.ReadJSON(&data); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if data.CustOrderId == "" {
			supports.Error(ctx, iris.StatusBadRequest, "未获取到客户订单ID", nil)
			return
		}
		if data.Qty == 0 {
			supports.Error(ctx, iris.StatusBadRequest, "获取到的取消数量为0", nil)
			return
		}
		err := method(user, data.CustOrderId, data.Qty)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}


/******************************************************************************
 *
 * @Function Name            : RegisterAnalysisExcelToCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 解析Excel生成客户订单
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-08
 *
 ******************************************************************************/
func RegisterAnalysisExcelToCustOrder(party router.Party, path string, method func(user *global.User, filepath string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		var err error = nil
		_, info, err := ctx.FormFile("uploadfile")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "获取上传文件出错,err: "+err.Error(), nil)
			return
		}

		guid := utils.MakeOrderSn("CustOrder")
		filenameLi := strings.Split(info.Filename, ".")
		filename := filenameLi[0]
		savePath := "web/public/uploadxlsx/" + filename + guid + ".xlsx"
		err = utils.SaveUploadedFile(info, savePath)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "保存上传文件出错,err: "+err.Error(), nil)
			return
		}

		err = method(user, savePath)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectReleaseCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 查询派工任务
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-08
 *
 ******************************************************************************/
func RegisterSelectReleaseCustOrder(party router.Party, path string, method func(user *global.User) ([]model.CustOrder, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		data, err := method(user)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, data)
	})
}



/******************************************************************************
 *
 * @Function Name            : RegisterSelectReleaseCustOrder
 *-----------------------------------------------------------------------------
 *
 * @Description              : 查询派工任务
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-03-08
 *
 ******************************************************************************/
func RegisterRepairReleaseFailedTask(party router.Party, path string, method func(user *global.User) error) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		err := method(user)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}