// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package pln

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/pln"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条HondaCalloffErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterOneHondaCalloffErrorLst(party router.Party, httpMethod string, path string, method func(*global.User, *model.HondaCalloffErrorLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil

		entity := new(model.HondaCalloffErrorLst)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条HondaCalloffErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterIDOfHondaCalloffErrorLst(party router.Party, httpMethod string, path string, method func(*global.User, string, int) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		calloffId := ctx.Params().GetString("calloffId")
		pos, err := ctx.Params().GetInt("pos")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, calloffId, pos)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条HondaCalloffErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterInsertOneHondaCalloffErrorLst(party router.Party, path string, method func(*global.User, *model.HondaCalloffErrorLst) error) {

	RegisterOneHondaCalloffErrorLst(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条HondaCalloffErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterDeleteOneHondaCalloffErrorLst(party router.Party, path string, method func(*global.User, string, int) error) {

	RegisterIDOfHondaCalloffErrorLst(party, "DELETE", path+"/{calloffId:string}/{pos:int}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条HondaCalloffErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterSelectOneHondaCalloffErrorLst(party router.Party, path string, method func(*global.User, string, int) (*model.HondaCalloffErrorLst, error)) {

	party.Get(path+"/{calloffId:string}/{pos:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		calloffId := ctx.Params().GetString("calloffId")
		pos, err := ctx.Params().GetInt("pos")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		result, err := method(user, calloffId, pos)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条HondaCalloffErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterUpdateOneHondaCalloffErrorLst(party router.Party, path string, method func(*global.User, *model.HondaCalloffErrorLst) error) {

	RegisterOneHondaCalloffErrorLst(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条HondaCalloffErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterMultiHondaCalloffErrorLst(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.HondaCalloffErrorLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		entities := make([]model.HondaCalloffErrorLst, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条HondaCalloffErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterInsertHondaCalloffErrorLst(party router.Party, path string, method func(*global.User, *[]model.HondaCalloffErrorLst) error) {

	RegisterMultiHondaCalloffErrorLst(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条HondaCalloffErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterDeleteHondaCalloffErrorLst(party router.Party, path string, method func(*global.User, *[]model.HondaCalloffErrorLst) error) {

	RegisterMultiHondaCalloffErrorLst(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询HondaCalloffErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterSelectHondaCalloffErrorLst(party router.Party, path string, method func(*global.User, map[string]string) ([]model.HondaCalloffErrorLst, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询HondaCalloffErrorLst并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterQueryHondaCalloffErrorLst(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条HondaCalloffErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterUpdateHondaCalloffErrorLst(party router.Party, path string, method func(*global.User, *[]model.HondaCalloffErrorLst) error) {

	RegisterMultiHondaCalloffErrorLst(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneHondaCalloffErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条HondaCalloffErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-11-09 13:35:31
 *
 ******************************************************************************/
func RegisterRepairError(party router.Party, path string, method func(*global.User, string, int) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		type Req struct {
			CallOffId string `json:"calloffId"`
			Pos int `json:"pos"`
		}
		var data Req
		if err := ctx.ReadJSON(&data); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err := method(user, data.CallOffId, data.Pos)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterRepairAllError
 *-----------------------------------------------------------------------------
 *
 * @Description              : 修复所有错误
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 张鑫
 *
 * @Date                     : 2022-02-09
 *
 ******************************************************************************/
func RegisterRepairAllError(party router.Party, path string, method func(*global.User, string) error) {

	party.Post(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		type Req struct {
			CallOffId string `json:"calloffId"`
		}
		var data Req
		if err := ctx.ReadJSON(&data); err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err := method(user, data.CallOffId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}