// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package pln

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/pln"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneHondaDemandErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条HondaDemandErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-20 09:58:33
 *
 ******************************************************************************/
func RegisterOneHondaDemandErrorLst(party router.Party, httpMethod string, path string, method func(*global.User, *model.HondaDemandErrorLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil

		entity := new(model.HondaDemandErrorLst)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfHondaDemandErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条HondaDemandErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-20 09:58:33
 *
 ******************************************************************************/
func RegisterIDOfHondaDemandErrorLst(party router.Party, httpMethod string, path string, method func(*global.User, string, int) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		demandId := ctx.Params().GetString("demandId")
		pos, err := ctx.Params().GetInt("pos")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, demandId, pos)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneHondaDemandErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条HondaDemandErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-20 09:58:33
 *
 ******************************************************************************/
func RegisterInsertOneHondaDemandErrorLst(party router.Party, path string, method func(*global.User, *model.HondaDemandErrorLst) error) {

	RegisterOneHondaDemandErrorLst(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneHondaDemandErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条HondaDemandErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-20 09:58:33
 *
 ******************************************************************************/
func RegisterDeleteOneHondaDemandErrorLst(party router.Party, path string, method func(*global.User, string, int) error) {

	RegisterIDOfHondaDemandErrorLst(party, "DELETE", path+"/{demandId:string}/{pos:int}", method)
}

func RegisterRepairHondaDemandErrorLst(party router.Party, path string, method func(*global.User, string, int) error) {
	party.Handle("PUT", path+"/{demandId:string}/{pos:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		demandId := ctx.Params().GetString("demandId")
		pos, err := ctx.Params().GetInt("pos")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, demandId, pos)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneHondaDemandErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条HondaDemandErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-20 09:58:33
 *
 ******************************************************************************/
func RegisterSelectOneHondaDemandErrorLst(party router.Party, path string, method func(*global.User, string, int) (*model.HondaDemandErrorLst, error)) {

	party.Get(path+"/{demandId:string}/{pos:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		demandId := ctx.Params().GetString("demandId")
		pos, err := ctx.Params().GetInt("pos")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		result, err := method(user, demandId, pos)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneHondaDemandErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条HondaDemandErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-20 09:58:33
 *
 ******************************************************************************/
func RegisterUpdateOneHondaDemandErrorLst(party router.Party, path string, method func(*global.User, *model.HondaDemandErrorLst) error) {

	RegisterOneHondaDemandErrorLst(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiHondaDemandErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条HondaDemandErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-20 09:58:33
 *
 ******************************************************************************/
func RegisterMultiHondaDemandErrorLst(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.HondaDemandErrorLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		entities := make([]model.HondaDemandErrorLst, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertHondaDemandErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条HondaDemandErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-20 09:58:33
 *
 ******************************************************************************/
func RegisterInsertHondaDemandErrorLst(party router.Party, path string, method func(*global.User, *[]model.HondaDemandErrorLst) error) {

	RegisterMultiHondaDemandErrorLst(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteHondaDemandErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条HondaDemandErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-20 09:58:33
 *
 ******************************************************************************/
func RegisterDeleteHondaDemandErrorLst(party router.Party, path string, method func(*global.User, *[]model.HondaDemandErrorLst) error) {

	RegisterMultiHondaDemandErrorLst(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectHondaDemandErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询HondaDemandErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-20 09:58:33
 *
 ******************************************************************************/
func RegisterSelectHondaDemandErrorLst(party router.Party, path string, method func(*global.User, map[string]string) ([]model.HondaDemandErrorLst, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryHondaDemandErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询HondaDemandErrorLst并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-20 09:58:33
 *
 ******************************************************************************/
func RegisterQueryHondaDemandErrorLst(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateHondaDemandErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条HondaDemandErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-20 09:58:33
 *
 ******************************************************************************/
func RegisterUpdateHondaDemandErrorLst(party router.Party, path string, method func(*global.User, *[]model.HondaDemandErrorLst) error) {

	RegisterMultiHondaDemandErrorLst(party, "PUT", path, method)
}
