// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package pln

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/pln"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条ToyotaDeliveryOrderErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterOneToyotaDeliveryOrderErrorLst(party router.Party, httpMethod string, path string, method func(*global.User, *model.ToyotaDeliveryOrderErrorLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil

		entity := new(model.ToyotaDeliveryOrderErrorLst)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条ToyotaDeliveryOrderErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterIDOfToyotaDeliveryOrderErrorLst(party router.Party, httpMethod string, path string, method func(*global.User, string, int) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		var err error = nil
		demandId := ctx.Params().GetString("demandId")
		pos, err := ctx.Params().GetInt("pos")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, demandId, pos)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条ToyotaDeliveryOrderErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterInsertOneToyotaDeliveryOrderErrorLst(party router.Party, path string, method func(*global.User, *model.ToyotaDeliveryOrderErrorLst) error) {

	RegisterOneToyotaDeliveryOrderErrorLst(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条ToyotaDeliveryOrderErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterDeleteOneToyotaDeliveryOrderErrorLst(party router.Party, path string, method func(*global.User, string, int) error) {

	RegisterIDOfToyotaDeliveryOrderErrorLst(party, "DELETE", path+"/{demandId:string}/{pos:int}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条ToyotaDeliveryOrderErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterSelectOneToyotaDeliveryOrderErrorLst(party router.Party, path string, method func(*global.User, string, int) (*model.ToyotaDeliveryOrderErrorLst, error)) {

	party.Get(path+"/{demandId:string}/{pos:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		demandId := ctx.Params().GetString("demandId")
		pos, err := ctx.Params().GetInt("pos")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		result, err := method(user, demandId, pos)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条ToyotaDeliveryOrderErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterUpdateOneToyotaDeliveryOrderErrorLst(party router.Party, path string, method func(*global.User, *model.ToyotaDeliveryOrderErrorLst) error) {

	RegisterOneToyotaDeliveryOrderErrorLst(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条ToyotaDeliveryOrderErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterMultiToyotaDeliveryOrderErrorLst(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.ToyotaDeliveryOrderErrorLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		entities := make([]model.ToyotaDeliveryOrderErrorLst, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条ToyotaDeliveryOrderErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterInsertToyotaDeliveryOrderErrorLst(party router.Party, path string, method func(*global.User, *[]model.ToyotaDeliveryOrderErrorLst) error) {

	RegisterMultiToyotaDeliveryOrderErrorLst(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条ToyotaDeliveryOrderErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterDeleteToyotaDeliveryOrderErrorLst(party router.Party, path string, method func(*global.User, *[]model.ToyotaDeliveryOrderErrorLst) error) {

	RegisterMultiToyotaDeliveryOrderErrorLst(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询ToyotaDeliveryOrderErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterSelectToyotaDeliveryOrderErrorLst(party router.Party, path string, method func(*global.User, map[string]string) ([]model.ToyotaDeliveryOrderErrorLst, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询ToyotaDeliveryOrderErrorLst并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterQueryToyotaDeliveryOrderErrorLst(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条ToyotaDeliveryOrderErrorLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterUpdateToyotaDeliveryOrderErrorLst(party router.Party, path string, method func(*global.User, *[]model.ToyotaDeliveryOrderErrorLst) error) {

	RegisterMultiToyotaDeliveryOrderErrorLst(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryToyotaDeliveryOrderErrorLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询ToyotaDeliveryOrderErrorLst并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2022-02-16 10:14:04
 *
 ******************************************************************************/
func RegisterSelectError(party router.Party, path string, method func(user *global.User, demandId string) (*model.ToyotaDeliveryOrderErrorLst, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}
		demandId := ctx.URLParam("demandId")
		result, err := method(user, demandId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}