// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package pm

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/pm"
	"LAPP_ACURA_MOM_BACKEND/utils"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	json "github.com/json-iterator/go"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
	"log"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneWorkOrderAttrLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条WorkOrderAttrLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterOneWorkOrderAttrLst(party router.Party, httpMethod string, path string, method func(*global.User, *model.WorkOrderAttrLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.WorkOrderAttrLst)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfWorkOrderAttrLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条WorkOrderAttrLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterIDOfWorkOrderAttrLst(party router.Party, httpMethod string, path string, method func(*global.User, string, int, int) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		maintWorkOrderId := ctx.Params().GetString("maintWorkOrderId")
		mainServiceNr, err := ctx.Params().GetInt("mainServiceNr")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		attrNr, err := ctx.Params().GetInt("attrNr")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, maintWorkOrderId, mainServiceNr, attrNr)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneWorkOrderAttrLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条WorkOrderAttrLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterInsertOneWorkOrderAttrLst(party router.Party, path string, method func(*global.User, *model.WorkOrderAttrLst) error) {

	RegisterOneWorkOrderAttrLst(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneWorkOrderAttrLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条WorkOrderAttrLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterDeleteOneWorkOrderAttrLst(party router.Party, path string, method func(*global.User, string, int, int) error) {

	RegisterIDOfWorkOrderAttrLst(party, "DELETE", path+"/{maintWorkOrderId:string}/{mainServiceNr:int}/{attrNr:int}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneWorkOrderAttrLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条WorkOrderAttrLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterSelectOneWorkOrderAttrLst(party router.Party, path string, method func(*global.User, string, int, int) (*model.WorkOrderAttrLst, error)) {

	party.Get(path+"/{maintWorkOrderId:string}/{mainServiceNr:int}/{attrNr:int}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		maintWorkOrderId := ctx.Params().GetString("maintWorkOrderId")
		mainServiceNr, err := ctx.Params().GetInt("mainServiceNr")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		attrNr, err := ctx.Params().GetInt("attrNr")
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		result, err := method(user, maintWorkOrderId, mainServiceNr, attrNr)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneWorkOrderAttrLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条WorkOrderAttrLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterUpdateOneWorkOrderAttrLst(party router.Party, path string, method func(*global.User, *model.WorkOrderAttrLst) error) {

	RegisterOneWorkOrderAttrLst(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiWorkOrderAttrLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条WorkOrderAttrLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterMultiWorkOrderAttrLst(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.WorkOrderAttrLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.WorkOrderAttrLst, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertWorkOrderAttrLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条WorkOrderAttrLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterInsertWorkOrderAttrLst(party router.Party, path string, method func(*global.User, *[]model.WorkOrderAttrLst) error) {

	RegisterMultiWorkOrderAttrLst(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteWorkOrderAttrLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条WorkOrderAttrLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterDeleteWorkOrderAttrLst(party router.Party, path string, method func(*global.User, *[]model.WorkOrderAttrLst) error) {

	RegisterMultiWorkOrderAttrLst(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectWorkOrderAttrLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询WorkOrderAttrLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterSelectWorkOrderAttrLst(party router.Party, path string, method func(*global.User, map[string]string) ([]model.WorkOrderAttrLst, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryWorkOrderAttrLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询WorkOrderAttrLst并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterQueryWorkOrderAttrLst(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateWorkOrderAttrLst
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条WorkOrderAttrLst的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterUpdateWorkOrderAttrLst(party router.Party, path string, method func(*global.User, *[]model.WorkOrderAttrLst) error) {

	RegisterMultiWorkOrderAttrLst(party, "PUT", path, method)
}

func RegisterUploadWorkOrderAttrLst(party router.Party, path string, method func(*global.User, *model.WorkOrderAttrLst) error) {

	RegisterMultiUploadWorkOrderAttrLst(party, "POST", path, method)
}

func RegisterMultiUploadWorkOrderAttrLst(party router.Party, httpMethod string, path string, method func(*global.User, *model.WorkOrderAttrLst) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		//获取通过iris.WithPostMaxMemory获取的最大上传值大小。
		maxSize := ctx.Application().ConfigurationReadOnly().GetPostMaxMemory()
		err := ctx.Request().ParseMultipartForm(maxSize)
		if err != nil {
			ctx.StatusCode(iris.StatusInternalServerError)
			ctx.WriteString(err.Error())
			return
		}
		form := ctx.Request().MultipartForm
		files := form.File["upload[]"]

		entity := new(model.WorkOrderAttrLst)
		info := ctx.FormValue("data")
		err = json.Unmarshal([]byte(info), entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "json解析错误:"+err.Error(), nil)
			return
		}

		if entity == nil {
			supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到原有信息:"+err.Error(), nil)
			return
		}

		for _, file := range files {
			if !utils.ValueIsEmpty(file.Filename) {
				guid := utils.MakeOrderSn(user.UserId)
				filPath := "public/uploadfile/" + guid + ".jpg"
				savePath := "web/public/uploadfile/" + guid + ".jpg"
				// 上传文件至指定目录
				err := utils.SaveUploadedFile(file, savePath)
				if err != nil {
					log.Printf("上传图片错误:%v", err)
					supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
					return
				}
				entity.Bitmap = filPath
			}
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}
