// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package qm

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	model "LAPP_ACURA_MOM_BACKEND/models/qm"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"

	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneSerialOrderRuting
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条SerialOrderRuting的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-25 11:18:25
 *
 ******************************************************************************/
func RegisterOneSerialOrderRuting(party router.Party, httpMethod string, path string, method func(*global.User, *model.SerialOrderRuting) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.SerialOrderRuting)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfSerialOrderRuting
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条SerialOrderRuting的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-25 11:18:25
 *
 ******************************************************************************/
func RegisterIDOfSerialOrderRuting(party router.Party, httpMethod string, path string, method func(*global.User, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		serialorderId := ctx.Params().GetString("serialorderId")

		err = method(user, serialorderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneSerialOrderRuting
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条SerialOrderRuting的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-25 11:18:25
 *
 ******************************************************************************/
func RegisterInsertOneSerialOrderRuting(party router.Party, path string, method func(*global.User, *model.SerialOrderRuting) error) {

	RegisterOneSerialOrderRuting(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneSerialOrderRuting
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条SerialOrderRuting的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-25 11:18:25
 *
 ******************************************************************************/
func RegisterDeleteOneSerialOrderRuting(party router.Party, path string, method func(*global.User, string) error) {

	RegisterIDOfSerialOrderRuting(party, "DELETE", path+"/{serialorderId:string}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneSerialOrderRuting
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条SerialOrderRuting的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-25 11:18:25
 *
 ******************************************************************************/
func RegisterSelectOneSerialOrderRuting(party router.Party, path string, method func(*global.User, string) (*model.SerialOrderRuting, error)) {

	party.Get(path+"/{serialorderId:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		serialorderId := ctx.Params().GetString("serialorderId")
		result, err := method(user, serialorderId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneSerialOrderRuting
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条SerialOrderRuting的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-25 11:18:25
 *
 ******************************************************************************/
func RegisterUpdateOneSerialOrderRuting(party router.Party, path string, method func(*global.User, *model.SerialOrderRuting) error) {

	RegisterOneSerialOrderRuting(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiSerialOrderRuting
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条SerialOrderRuting的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-25 11:18:25
 *
 ******************************************************************************/
func RegisterMultiSerialOrderRuting(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.SerialOrderRuting) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.SerialOrderRuting, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertSerialOrderRuting
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条SerialOrderRuting的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-25 11:18:25
 *
 ******************************************************************************/
func RegisterInsertSerialOrderRuting(party router.Party, path string, method func(*global.User, *[]model.SerialOrderRuting) error) {

	RegisterMultiSerialOrderRuting(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteSerialOrderRuting
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条SerialOrderRuting的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-25 11:18:25
 *
 ******************************************************************************/
func RegisterDeleteSerialOrderRuting(party router.Party, path string, method func(*global.User, *[]model.SerialOrderRuting) error) {

	RegisterMultiSerialOrderRuting(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectSerialOrderRuting
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询SerialOrderRuting的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-25 11:18:25
 *
 ******************************************************************************/
func RegisterSelectSerialOrderRuting(party router.Party, path string, method func(*global.User, map[string]string) (interface{}, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateSerialOrderRuting
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条SerialOrderRuting的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-06-25 11:18:25
 *
 ******************************************************************************/
func RegisterUpdateSerialOrderRuting(party router.Party, path string, method func(*global.User, *[]model.SerialOrderRuting) error) {

	RegisterMultiSerialOrderRuting(party, "PUT", path, method)
}
