// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package wm

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	model "LAPP_ACURA_MOM_BACKEND/models/wm"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneStorageRack
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条StorageRack的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-05 10:40:38
 *
 ******************************************************************************/
func RegisterOneStorageRack(party router.Party, httpMethod string, path string, method func(*global.User, *model.StorageRack) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil

		entity := new(model.StorageRack)
		if ctx.GetContentLength() > 0 {
			err := ctx.ReadJSON(entity)
			if err != nil {
				supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
				return
			}
		} else {
			entity = nil
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfStorageRack
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条StorageRack的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-05 10:40:38
 *
 ******************************************************************************/
func RegisterIDOfStorageRack(party router.Party, httpMethod string, path string, method func(*global.User, string, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		storageRackId := ctx.Params().GetString("storageRackId")
		warehouseId := ctx.Params().GetString("warehouseId")

		err = method(user, storageRackId, warehouseId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneStorageRack
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条StorageRack的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-05 10:40:38
 *
 ******************************************************************************/
func RegisterInsertOneStorageRack(party router.Party, path string, method func(*global.User, *model.StorageRack) error) {

	RegisterOneStorageRack(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneStorageRack
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条StorageRack的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-05 10:40:38
 *
 ******************************************************************************/
func RegisterDeleteOneStorageRack(party router.Party, path string, method func(*global.User, string, string) error) {

	RegisterIDOfStorageRack(party, "DELETE", path+"/{storageRackId:string}/{warehouseId:string}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneStorageRack
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条StorageRack的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-05 10:40:38
 *
 ******************************************************************************/
func RegisterSelectOneStorageRack(party router.Party, path string, method func(*global.User, string, string) (*model.StorageRack, error)) {

	party.Get(path+"/{storageRackId:string}/{warehouseId:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		storageRackId := ctx.Params().GetString("storageRackId")
		warehouseId := ctx.Params().GetString("warehouseId")
		result, err := method(user, storageRackId, warehouseId)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneStorageRack
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条StorageRack的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-05 10:40:38
 *
 ******************************************************************************/
func RegisterUpdateOneStorageRack(party router.Party, path string, method func(*global.User, *model.StorageRack) error) {

	RegisterOneStorageRack(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiStorageRack
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条StorageRack的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-05 10:40:38
 *
 ******************************************************************************/
func RegisterMultiStorageRack(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.StorageRack) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.StorageRack, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertStorageRack
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条StorageRack的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-05 10:40:38
 *
 ******************************************************************************/
func RegisterInsertStorageRack(party router.Party, path string, method func(*global.User, *[]model.StorageRack) error) {

	RegisterMultiStorageRack(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteStorageRack
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条StorageRack的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-05 10:40:38
 *
 ******************************************************************************/
func RegisterDeleteStorageRack(party router.Party, path string, method func(*global.User, *[]model.StorageRack) error) {

	RegisterMultiStorageRack(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectStorageRack
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询StorageRack的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-05 10:40:38
 *
 ******************************************************************************/
func RegisterSelectStorageRack(party router.Party, path string, method func(*global.User, map[string]string) (interface{}, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateStorageRack
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条StorageRack的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-07-05 10:40:38
 *
 ******************************************************************************/
func RegisterUpdateStorageRack(party router.Party, path string, method func(*global.User, *[]model.StorageRack) error) {

	RegisterMultiStorageRack(party, "PUT", path, method)
}
